﻿using System.ComponentModel.DataAnnotations;
using System.Security.Claims;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.RazorPages;
using Microsoft.Extensions.Options;

namespace HMI_Base.Areas.Login.Pages;

public class IndexModel(IOptions<AuthenticationSettings> authSettings) : PageModel
{
    private readonly AuthenticationSettings _authSettings = authSettings.Value;

    [BindProperty]
    [Required(ErrorMessage = "ユーザー名を入力してください。")]
    public string? UserName { get; set; }

    [BindProperty]
    [Required(ErrorMessage = "パスワードを入力してください。")]
    public string? Password { get; set; }

    public IActionResult OnGet()
    {
        // ログインが無効であれば、ホームページにリダイレクト
        if (!_authSettings.IsLoginEnabled)
        {
            return Redirect("/");
        }
        return Page();
    }

    public async Task<IActionResult> OnPost()
    {
        if (ModelState.IsValid is false) return Page();

        // ユーザー名とパスワードが一致するかを確認
        if (UserName == _authSettings.AllowedUserName && Password == _authSettings.AllowedPassword)
        {
            var claims = new[]
            {
                new Claim(ClaimTypes.Name, UserName!),
                new Claim(ClaimTypes.Role, "User"),
                new Claim(ClaimTypes.Role, "Administrator")
            };

            var principal = new ClaimsPrincipal(new ClaimsIdentity(claims, CookieAuthenticationDefaults.AuthenticationScheme));

            // サインイン処理
            await HttpContext.SignInAsync(CookieAuthenticationDefaults.AuthenticationScheme, principal);

            return Redirect("/"); // ログイン成功後のリダイレクト先
        }
        else
        {
            // ユーザー名またはパスワードが一致しない場合のエラーメッセージ
            ModelState.AddModelError(string.Empty, "ユーザー名またはパスワードが正しくありません。");
            return Page();
        }
    }
}