﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using Microsoft.AspNetCore.Components.Server.Circuits;
using System;
using System.Collections.Concurrent;
using System.Threading;
using System.Threading.Tasks;

public class ClientConnectionHandler : CircuitHandler
{
    // サーキットIDをキーとして、サーキット閉鎖時のクリーンアップ処理を管理するための ConcurrentDictionary
    private readonly ConcurrentDictionary<string, Action> circuitCleanupActions = new();

    // サーキットIDを関連付けるための ConcurrentDictionary（現在は使用されていない）
    private readonly ConcurrentDictionary<string, string> circuitIds = new();

    // 現在のサーキットIDを保持
    public string NowCircuitId = "";

    // サーキットが開かれた際に呼び出される非同期メソッド
    public override Task OnCircuitOpenedAsync(Circuit circuit, CancellationToken cancellationToken)
    {
        // 現在のサーキットIDを保持
        NowCircuitId = circuit.Id;
        // サーキットオープン時の処理
        Console.WriteLine($"Circuit opened: {NowCircuitId}");

        // 完了したタスクを返す（サーキットオープン時の処理が完了）
        return Task.CompletedTask;
    }

    // サーキットが閉じられた際に呼び出される非同期メソッド
    public override Task OnCircuitClosedAsync(Circuit circuit, CancellationToken cancellationToken)
    {
        // サーキットが閉じられたことをコンソールに出力
        Console.WriteLine($"Circuit closed: {circuit.Id}");

        // サーキットが閉じられた際にクリーンアップ処理を実行
        // サーキットIDに対応するクリーンアップアクションがあれば実行
        if (circuitCleanupActions.TryRemove(circuit.Id, out var cleanupAction))
        {
            cleanupAction?.Invoke(); // クリーンアップ処理を実行
        }

        // サーキットクローズ時の処理が完了したことを示す
        return Task.CompletedTask;
    }

    // サーキットIDに関連付けてクリーンアップアクションを登録するメソッド
    public void RegisterCleanupAction(string circuitId, Action cleanupAction)
    {
        // サーキットIDに対応するクリーンアップアクションを登録
        circuitCleanupActions[circuitId] = cleanupAction;
    }
}
