﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using HMI_Container;
using Microsoft.AspNetCore.Http.HttpResults;
using System;
using System.IO.Pipelines;
using System.Xml.Linq;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using System.Data;
using System.Collections.Generic;

// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が無効になる(従来通り)
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif

namespace HMI_Base
{
    /// <summary>
    /// Edge関連IF
    /// </summary>
    //public class EdgeIF : IDisposable
    public class EdgeIF
    {
        /// <summary>
        /// RT-edge API
        /// </summary>
        //edge_API EGAPI = new edge_API();

        /// <summary>
        /// HMIクライアントクラス
        /// </summary>
        HMI_Client m_clClient = null;

        // EgLog出力用のログタイプ定義
        // 定数定義
        static public readonly int LOG_MODE_OFF = 0x0000;
        static public readonly int LOG_MODE_ERR = 0x0001;
        static public readonly int LOG_MODE_DEBUG = 0x0002;

        static public readonly string LOG_ERROR = "ERROR";
        static public readonly string LOG_DEBUG = "DEBUG";

        //タスクキャンセル用
        //private readonly CancellationTokenSource _cts = new();


        private readonly ILogger<EdgeIF> _logger;
        private readonly AppSettings _appSettings;
        int inConnectTimeout = 1000;// ★TODO 設定で変えられるように
        public EdgeIF(ILogger<EdgeIF> logger, IOptions<AppSettings> appSettings)
        {
            _logger = logger;
            _appSettings = appSettings.Value;  // 実際の AppSettings インスタンスを取得
            //Task.Run(() => EdgeIFTask(_cts.Token));

            //Serverとのやり取りを行うClientクラスの初期化
            m_clClient = new HMI_Client();
            
            LOG(EdgeIF.LOG_DEBUG, "EdgeIF Constructor Call");
        }
        ~EdgeIF()
        {
            Dispose();
            LOG(EdgeIF.LOG_DEBUG, "EdgeIF Destructor Call");
        }

        /// <summary>
        /// 破棄
        /// </summary>
        public void Dispose()
        {
            //クライアントクラスの破棄
            m_clClient.Dispose();
        }
        //private void EdgeIFTask(CancellationToken cancellationToken)
        //{
        //    while (!cancellationToken.IsCancellationRequested)
        //    {
        //        // タスク内の処理

        //    }
        //}
        /// <summary>
        /// 接続チェック 接続していなかったら接続する
        /// </summary>
        /// <returns></returns>
        private UInt32 ConnectINCheck()
        {
            try
            {
                string srMsg = "";
                UInt32 ret = 0;
                // m_clPipeInがNULLの場合はopen処理
                if (m_clClient.m_clPipeIn == null)
                {
                    //入力処理通信(Pipe)のオープン
                    ret = m_clClient.OpenIn(ref srMsg);
                    if (ret != (UInt32)HMI_Static.ErrCode.Ok) //失敗時は終了
                    {
                        LOG(EdgeIF.LOG_ERROR, "m_clClient.OpenIn ERROR : ID = " + ret.ToString() + " , Msg = " + srMsg);
                        return ret;
                    }
                }
                
                // 生成できない場合は処理を抜ける
                if (m_clClient.m_clPipeIn == null)
                    return (UInt32)HMI_Static.ErrCode.NotCreate;

                // m_clPipeInが未接続の場合は接続処理
                if (m_clClient.m_clPipeIn.IsConnected == false)
                {
                    //接続処理実行
                    ret = m_clClient.ConnectIn(inConnectTimeout, ref srMsg);
                    if (ret != (UInt32)HMI_Static.ErrCode.Ok) //失敗時は終了
                    {
                        LOG(EdgeIF.LOG_ERROR, "m_clClient.ConnectIn ERROR : ID = " + ret.ToString() + " , Msg = " + srMsg);
                        return ret;
                    }
                }
                
                // 未接続の場合は処理を抜ける
                if (m_clClient.m_clPipeIn.IsConnected == false)
                {
                    LOG(EdgeIF.LOG_ERROR, "ConnectCheck Not Connected");
                    return (UInt32)HMI_Static.ErrCode.NotConnect;
                }

                return (UInt32)HMI_Static.ErrCode.Ok;
                
            }
            catch (System.Exception ex)
            {
                LOG(EdgeIF.LOG_ERROR,   "ConnectCheck Fatal Error : " + ex.Message);
                return (UInt32)ex.HResult;
                
            }
        }
        private UInt32 ConnectOUTCheck()
        {
            try
            {
                string srMsg = "";
                UInt32 ret = 0;
                // m_clPipeOutがNULLの場合はopen処理
                if (m_clClient.m_clPipeOut == null)
                {
                    //出力処理通信(Pipe)のオープン
                    ret = m_clClient.OpenOut(ref srMsg);
                    if (ret != (UInt32)HMI_Static.ErrCode.Ok) //失敗時は終了
                    {
                        LOG(EdgeIF.LOG_ERROR, "m_clClient.OpenOut ERROR : ID = " + ret.ToString() + " , Msg = " + srMsg);
                        return ret;
                    }
                }
                // 生成できない場合は処理を抜ける
                if (m_clClient.m_clPipeOut == null)
                    return (UInt32)HMI_Static.ErrCode.NotCreate;

                // m_clPipeOutが未接続の場合は接続処理
                if (m_clClient.m_clPipeOut.IsConnected == false)
                {
                    //接続処理実行
                    ret = m_clClient.ConnectOut(inConnectTimeout, ref srMsg);
                    if (ret != (UInt32)HMI_Static.ErrCode.Ok) //失敗時は終了
                    {
                        LOG(EdgeIF.LOG_ERROR, "m_clClient.ConnectOut ERROR : ID = " + ret.ToString() + " , Msg = " + srMsg);
                        return ret;
                    }
                }
                // 未接続の場合は処理を抜ける
                if (m_clClient.m_clPipeOut.IsConnected == false)
                {
                    LOG(EdgeIF.LOG_ERROR,   "ConnectCheck Not Connected");
                    return (UInt32)HMI_Static.ErrCode.NotConnect;
                }

                return (UInt32)HMI_Static.ErrCode.Ok;

            }
            catch (System.Exception ex)
            {
                LOG(EdgeIF.LOG_ERROR, "ConnectCheck Fatal Error : " + ex.Message);
                return (UInt32)ex.HResult;

            }
        }

        /// <summary>
        /// タグの値取得 一つ
        /// </summary>
        /// <param name="name">タグ名</param>
        /// <returns>タグの値</returns>
        public object GetTag(string srName, ref ushort woType)
        {
            try
            {
                object retObject = null;
                
                //排他制御を行いながら処理を行う
                using (Mutex mutex = new Mutex(false, HMI_Static.HMI_PIPE_IN_BASE_MUTEX_NAME))
                {
                    // Mutexを取得する（他のプロセスが使っていれば待機）
                    mutex.WaitOne();
                    try
                    {
                        m_clClient.m_dwLastErr = 0; //初期化

                        //接続チェック & 接続処理
                        if (ConnectINCheck() == (UInt32)HMI_Static.ErrCode.Ok)
                        {

                            // 要求伝文(データ部)の生成
                            m_clClient.AddReqReadTagDataTag(srName);
                            // タグ読み込み処理
                            m_clClient.EntryReadTag();

                            // エラーがある場合は処理終了
                            if (m_clClient.m_dwLastErr != 0 || m_clClient.m_clMngMsgReadTag.m_clResData.m_lsTags.Count() == 0)
                            {
                                LOG(EdgeIF.LOG_ERROR, "GetTag ErrId :" + m_clClient.m_dwLastErr.ToString() + " , Message : " + m_clClient.m_srLastMsg);
                                m_clClient.CloseIn();
                            
                            }
                            else
                            {
                                woType = m_clClient.m_clMngMsgReadTag.m_clResData.m_lsTags[0].m_woType;
                                retObject = m_clClient.m_clMngMsgReadTag.m_clResData.m_lsTags[0].m_obVal;
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        LOG(EdgeIF.LOG_ERROR, "GetTag:" + srName + "  " + ex.Message);
                    }
                    finally
                    {
                        // Mutexの解放
                        mutex.ReleaseMutex();
                    }
                }
                return retObject;
            }
            catch (Exception ex)
            {
                LOG(EdgeIF.LOG_ERROR, "GetTag:" + srName + "  " + ex.Message);
                return null;
            }
        }
        /// <summary>
        /// タグの値を取得　リスト指定
        /// </summary>
        /// <param name="names"></param>
        /// <returns></returns>
        public List<MsgDataTagValue> GetTags(List<string> names)
        {
            try
            {

                List<MsgDataTagValue> objList = null;
                //排他制御を行いながら処理を行う
                using (Mutex mutex = new Mutex(false, HMI_Static.HMI_PIPE_IN_BASE_MUTEX_NAME))
                {
                    // Mutexを取得する（他のプロセスが使っていれば待機）
                    mutex.WaitOne();
                    try
                    {
                        m_clClient.m_dwLastErr = 0; //初期化

                        //接続チェック & 接続処理
                        if (ConnectINCheck() == (UInt32)HMI_Static.ErrCode.Ok)
                        {


                            // 要求伝文(データ部)の生成
                            m_clClient.AddReqReadTagDataTags(names);
                            // タグ読み込み処理
                            m_clClient.EntryReadTag();
                            // エラーがあったら値は無視
                            if (m_clClient.m_dwLastErr != 0 || m_clClient.m_clMngMsgReadTag.m_clResData.m_lsTags.Count() == 0)
                            {
                                LOG(EdgeIF.LOG_ERROR, "GetTags ErrId :" + m_clClient.m_dwLastErr.ToString() + " , Message : " + m_clClient.m_srLastMsg);
                                m_clClient.CloseIn();
                            }
                            else
                                objList = m_clClient.m_clMngMsgReadTag.m_clResData.m_lsTags;
                        }
                    }
                    catch (Exception ex)
                    {
                        string str = "";
                        foreach (string name in names)
                        {
                            str += name + ",";
                        }
                        LOG(EdgeIF.LOG_ERROR, "GetTags:" + str + "  " + ex.Message);
                    }
                    finally
                    {
                        // Mutexの解放
                        mutex.ReleaseMutex();
                    }
                }

                return objList;
            }
            catch (Exception ex)
            {
                string str = "";
                foreach(string name in names)
                {
                    str += name + ",";
                }
                LOG(EdgeIF.LOG_ERROR, "GetTags:" + str + "  " + ex.Message);
                return null;
            }
        }

        /// <summary>
        /// タグのセット関数
        /// </summary>
        /// <param name="name">タグ名</param>
        /// <param name="objVal">値</param>
        /// <returns>ステータス</returns>
        public int SetTag(string name, int objType, object objVal)
        {
            int ret = -1;
            try
            {
                //引数がおかしい
                if (name.Length == 0 || objVal == null)
                    return (int)HMI_Static.ErrCode.ArgumentIsInvalid;

                //排他制御を行いながら処理を行う
                using (Mutex mutex = new Mutex(false, HMI_Static.HMI_PIPE_OUT_BASE_MUTEX_NAME))
                {
                    // Mutexを取得する（他のプロセスが使っていれば待機）
                    mutex.WaitOne();
                    try
                    {
                        m_clClient.m_dwLastErr = 0; //初期化
                        //接続チェック & 接続処理
                        ret = (int)ConnectOUTCheck();
                        if (ret != (int)HMI_Static.ErrCode.Ok)
                            return ret;

                        if(objType == 0 || objType > (int)egTagDataType.String)
                            return (int)HMI_Static.ErrCode.ArgumentIsInvalid;

                        m_clClient.AddReqWriteTagDataTag(name, (UInt16)objType, objVal);
                        
                        // タグ書き込み処理
                        ret = (int)m_clClient.EntryWriteTag();
                        // エラーがあったら値は無視
                        if (m_clClient.m_dwLastErr != 0)
                        {
                            LOG(EdgeIF.LOG_ERROR, "SetTag ErrId :" + m_clClient.m_dwLastErr.ToString() + " , Message : " + m_clClient.m_srLastMsg);
                            m_clClient.CloseOut();
                        }
                    }
                    catch (Exception ex)
                    {
                        LOG(EdgeIF.LOG_ERROR, "SetTag:" + name + "  " + ex.Message);
                    }
                    finally
                    {
                        
                        // Mutexの解放
                        mutex.ReleaseMutex();
                    }
                }
                return ret;
            }
            catch (Exception ex)
            {
                LOG(EdgeIF.LOG_ERROR, "SetTag:" + name + "  " + ex.Message);
                return -1;
            }
            finally
            {

            }
        }

        
        /// <summary>
        /// ログ出力
        /// </summary>
        /// <param name="msg"></param>
        /// <param name="mode"></param>
        public void LOG(string logType, string msg)
        {
            if (_appSettings == null)
                return;

            string strMsg = "<EDGEIF> : " + msg;

            // ログ種別に応じて出力を制御
            if (logType == LOG_ERROR && (_appSettings.OutputLogMode & LOG_MODE_ERR) == LOG_MODE_ERR)
            {
                // デバッグコンソール出力
                Console.WriteLine(strMsg);
                // ファイルへ出力
                if (_appSettings.OutputLogFile == true)
                    _logger.LogError(strMsg);
            }
            else if (logType == LOG_DEBUG&& (_appSettings.OutputLogMode & LOG_MODE_DEBUG) == LOG_MODE_DEBUG)
            {
                // デバッグコンソール出力
                Console.WriteLine(strMsg);
                // ファイルへ出力
                if (_appSettings.OutputLogFile == true)
                    _logger.LogInformation(strMsg);
            }
           
            //if (EgInitError == false)   // RT-edgeの初期化エラー時はログ出力も抑止します
            //{
            //    // EgLogへ出力
            //    //edge_FW._C_EgFW_PrintLog("HMI_Base", mode, strMsg);
            //}
        }
        /// <summary>
        /// Dispose呼び出し
        /// </summary>
        //public void Dispose()
        //{
        //    // リクエスト終了時に呼ばれる
        //    _cts.Cancel(); // タスクにキャンセルを通知
        //}

    }
}
// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が有効になる
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif
