﻿using System.Xml.Serialization;
using HMI_CommonLibrary;
using HMI_Base;

namespace HMI_Base
{
    /// <summary>
    /// 丸ボタン Lamp付き Tgl機能付き 要素
    /// </summary>
    /// <remarks>
    /// 属性名はXmlAttributeで定義します。
    /// </remarks>
    public class HMI_ButtonLampTglCircleElem : HMI_LampElem
    {
        /// <summary>
        /// ComponentType
        /// </summary>
        [XmlAttribute("ButtonType")]
        public int ButtonType = 2;


        /// <summary>
        /// フォント色 ON時
        /// </summary>
        [XmlAttribute("ForeColor_ON")]
        public string ForeColor_ON = "";
        /// <summary>
        /// フォント色 OFF時
        /// </summary>
        [XmlAttribute("ForeColor_OFF")]
        public string ForeColor_OFF = "";

        /// <summary>
        /// 背景色 ON時
        /// </summary>
        [XmlAttribute("BackColor_ON")]
        public string BackColor_ON = "";
        /// <summary>
        /// 背景色 OFF時
        /// </summary>
        [XmlAttribute("BackColor_OFF")]
        public string BackColor_OFF = "";

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_ButtonLampTglCircleElem()
        {
            // 初期設定
            FontSize = "20px";
            Left = "0px";
            Top = "0px";
            Width = "100px";
            Height = "100px";
            BackColor = "silver";
        }

    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLampTglCircleGreen_Xml
    {
        [XmlArrayItem("HMI_ButtonLampTglCircleGreen")]
        public List<HMI_ButtonLampTglCircleElem>? HMI_ButtonLampTglCircleGreens = null;
    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLampTglCircleRed_Xml
    {
        [XmlArrayItem("HMI_ButtonLampTglCircleRed")]
        public List<HMI_ButtonLampTglCircleElem>? HMI_ButtonLampTglCircleReds = null;
    }

    /// <summary>
    /// xml情報(青色)
    /// </summary>
    public class HMI_ButtonLampTglCircleBlue_Xml
    {
        [XmlArrayItem("HMI_ButtonLampTglCircleBlue")]
        public List<HMI_ButtonLampTglCircleElem>? HMI_ButtonLampTglCircleBlues = null;
    }

    /// <summary>
    /// ログ表示 xmlのロード
    /// </summary>
    public partial class HMI_LoadXml
    {
        /// <summary>
        /// ボタンランプ付き 丸 xmlロード 緑
        /// </summary>
        /// <param name="hmiButtonCircleGreenXml"></param>
        static public void Load_HMI_ButtonLampTglCircleGreen_Xml(string folderName, ref HMI_ButtonLampTglCircleGreen_Xml hmiButtonLampTglCircleGreenXml)
        {
            hmiButtonLampTglCircleGreenXml = LoadFromXml<HMI_ButtonLampTglCircleGreen_Xml>($"{folderName}/HMI_ButtonLampTglCircleGreen.xml");
            if (null != hmiButtonLampTglCircleGreenXml)
            {
                if (null != hmiButtonLampTglCircleGreenXml.HMI_ButtonLampTglCircleGreens)
                {
                    foreach (var Item in hmiButtonLampTglCircleGreenXml.HMI_ButtonLampTglCircleGreens)
                    {
                        // 文字色の設定
                        Item.ForeColor = "white";
                        Item.ForeColor_OFF = "Gray";
                        Item.ForeColor_ON = "White";
                        // 背景色の設定
                        Item.BackColor = "green";
                        Item.BackColor_OFF = "rgba(3, 56, 0, 100)";
                        Item.BackColor_ON = "rgba(0, 175, 80, 100)";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
        /// <summary>
        /// ボタンランプ付き 丸 xmlロード 赤
        /// </summary>
        /// <param name="hmiButtonLampTglCircleRedXml"></param>
        static public void Load_HMI_ButtonLampTglCircleRed_Xml(string folderName, ref HMI_ButtonLampTglCircleRed_Xml hmiButtonLampTglCircleRedXml)
        {
            hmiButtonLampTglCircleRedXml = LoadFromXml<HMI_ButtonLampTglCircleRed_Xml>($"{folderName}/HMI_ButtonLampTglCircleRed.xml");
            if (null != hmiButtonLampTglCircleRedXml)
            {
                if (null != hmiButtonLampTglCircleRedXml.HMI_ButtonLampTglCircleReds)
                {
                    foreach (var Item in hmiButtonLampTglCircleRedXml.HMI_ButtonLampTglCircleReds)
                    {
                        // 文字色の設定
                        Item.ForeColor = "white";
                        Item.ForeColor_OFF = "Gray";
                        Item.ForeColor_ON = "White";
                        // 背景色の設定
                        Item.BackColor = "red";
                        Item.BackColor_OFF = "rgba(85, 0, 0, 100)";
                        Item.BackColor_ON = "rgba(255, 0, 0, 100)";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
        /// <summary>
        /// ボタンランプ付き 丸 xmlロード ロイヤルブルー
        /// </summary>
        /// <param name="hmiButtonLampTglCircleBlueXml"></param>
        static public void Load_HMI_ButtonLampTglCircleBlue_Xml(string folderName, ref HMI_ButtonLampTglCircleBlue_Xml hmiButtonLampTglCircleBlueXml)
        {
            hmiButtonLampTglCircleBlueXml = LoadFromXml<HMI_ButtonLampTglCircleBlue_Xml>($"{folderName}/HMI_ButtonLampTglCircleBlue.xml");
            if (null != hmiButtonLampTglCircleBlueXml)
            {
                if (null != hmiButtonLampTglCircleBlueXml.HMI_ButtonLampTglCircleBlues)
                {
                    foreach (var Item in hmiButtonLampTglCircleBlueXml.HMI_ButtonLampTglCircleBlues)
                    {
                        // 文字色の設定
                        Item.ForeColor = "white";
                        Item.ForeColor_OFF = "Gray";
                        Item.ForeColor_ON = "White";
                        // 背景色の設定
                        Item.BackColor = "blue";
                        Item.BackColor_OFF = "rgba(0, 0, 150, 100)";
                        Item.BackColor_ON = "rgba(0, 100, 255, 100)";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
    }
}
