﻿using System.Xml.Serialization;
using HMI_CommonLibrary;
using HMI_Base;

namespace HMI_Base
{
    /// <summary>
    /// 四角ボタン Lamp付き Tgl機能付き 要素
    /// </summary>
    /// <remarks>
    /// 属性名はXmlAttributeで定義します。
    /// </remarks>
    public class HMI_ButtonLampTglRectElem : HMI_LampElem
    {
        /// <summary>
        /// ComponentType
        /// </summary>
        [XmlAttribute("ButtonType")]
        public int ButtonType = 2;


        /// <summary>
        /// フォント色 ON時
        /// </summary>
        [XmlAttribute("ForeColor_ON")]
        public string ForeColor_ON = "";
        /// <summary>
        /// フォント色 OFF時
        /// </summary>
        [XmlAttribute("ForeColor_OFF")]
        public string ForeColor_OFF = "";

        /// <summary>
        /// 背景色 ON時
        /// </summary>
        [XmlAttribute("BackColor_ON")]
        public string BackColor_ON = "";
        /// <summary>
        /// 背景色 OFF時
        /// </summary>
        [XmlAttribute("BackColor_OFF")]
        public string BackColor_OFF = "";

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_ButtonLampTglRectElem()
        {
            // 初期設定
            FontSize = "20px";
            Left = "0px";
            Top = "0px";
            Width = "200px";
            Height = "100px";
            BackColor = "silver";
        }

    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLampTglRectGreen_Xml
    {
        [XmlArrayItem("HMI_ButtonLampTglRectGreen")]
        public List<HMI_ButtonLampTglRectElem>? HMI_ButtonLampTglRectGreens = null;
    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLampTglRectRed_Xml
    {
        [XmlArrayItem("HMI_ButtonLampTglRectRed")]
        public List<HMI_ButtonLampTglRectElem>? HMI_ButtonLampTglRectReds = null;
    }

    /// <summary>
    /// xml情報(青色)
    /// </summary>
    public class HMI_ButtonLampTglRectBlue_Xml
    {
        [XmlArrayItem("HMI_ButtonLampTglRectBlue")]
        public List<HMI_ButtonLampTglRectElem>? HMI_ButtonLampTglRectBlues = null;
    }

    /// <summary>
    /// ログ表示 xmlのロード
    /// </summary>
    public partial class HMI_LoadXml
    {
        /// <summary>
        /// ボタンランプ付き 四角 xmlロード 緑
        /// </summary>
        /// <param name="hmiButtonRectGreenXml"></param>
        static public void Load_HMI_ButtonLampTglRectGreen_Xml(string folderName, ref HMI_ButtonLampTglRectGreen_Xml hmiButtonLampTglRectGreenXml)
        {
            hmiButtonLampTglRectGreenXml = LoadFromXml<HMI_ButtonLampTglRectGreen_Xml>($"{folderName}/HMI_ButtonLampTglRectGreen.xml");
            if (null != hmiButtonLampTglRectGreenXml)
            {
                if (null != hmiButtonLampTglRectGreenXml.HMI_ButtonLampTglRectGreens)
                {
                    foreach (var Item in hmiButtonLampTglRectGreenXml.HMI_ButtonLampTglRectGreens)
                    {
                        // 文字色の設定
                        Item.ForeColor = "white";
                        Item.ForeColor_OFF = "Gray";
                        Item.ForeColor_ON = "White";
                        // 背景色の設定
                        Item.BackColor = "green";
                        Item.BackColor_OFF = "rgba(3, 56, 0, 100)";
                        Item.BackColor_ON = "rgba(0, 175, 80, 100)";
                    }
                }
            }
        }
        /// <summary>
        /// ボタンランプ付き 四角 xmlロード 赤
        /// </summary>
        /// <param name="hmiButtonLampTglRectRedXml"></param>
        static public void Load_HMI_ButtonLampTglRectRed_Xml(string folderName, ref HMI_ButtonLampTglRectRed_Xml hmiButtonLampTglRectRedXml)
        {
            hmiButtonLampTglRectRedXml = LoadFromXml<HMI_ButtonLampTglRectRed_Xml>($"{folderName}/HMI_ButtonLampTglRectRed.xml");
            if (null != hmiButtonLampTglRectRedXml)
            {
                if (null != hmiButtonLampTglRectRedXml.HMI_ButtonLampTglRectReds)
                {
                    foreach (var Item in hmiButtonLampTglRectRedXml.HMI_ButtonLampTglRectReds)
                    {
                        // 文字色の設定
                        Item.ForeColor = "white";
                        Item.ForeColor_OFF = "Gray";
                        Item.ForeColor_ON = "White";
                        // 背景色の設定
                        Item.BackColor = "red";
                        Item.BackColor_OFF = "rgba(85, 0, 0, 100)";
                        Item.BackColor_ON = "rgba(255, 0, 0, 100)";
                    }
                }
            }
        }
        /// <summary>
        /// ボタンランプ付き 四角 xmlロード ロイヤルブルー
        /// </summary>
        /// <param name="hmiButtonLampTglRectBlueXml"></param>
        static public void Load_HMI_ButtonLampTglRectBlue_Xml(string folderName, ref HMI_ButtonLampTglRectBlue_Xml hmiButtonLampTglRectBlueXml)
        {
            hmiButtonLampTglRectBlueXml = LoadFromXml<HMI_ButtonLampTglRectBlue_Xml>($"{folderName}/HMI_ButtonLampTglRectBlue.xml");
            if (null != hmiButtonLampTglRectBlueXml)
            {
                if (null != hmiButtonLampTglRectBlueXml.HMI_ButtonLampTglRectBlues)
                {
                    foreach (var Item in hmiButtonLampTglRectBlueXml.HMI_ButtonLampTglRectBlues)
                    {
                        // 文字色の設定
                        Item.ForeColor = "white";
                        Item.ForeColor_OFF = "Gray";
                        Item.ForeColor_ON = "White";
                        // 背景色の設定
                        Item.BackColor = "blue";
                        Item.BackColor_OFF = "rgba(0, 0, 150, 100)";
                        Item.BackColor_ON = "rgba(0, 100, 255, 100)";
                    }
                }
            }
        }
    }
}
