﻿using System.Xml.Serialization;
using HMI_Base;

namespace HMI_Base
{
    /// <summary>
    /// 丸ボタン要素
    /// </summary>
    /// <remarks>
    /// 属性名はXmlAttributeで定義します。
    /// </remarks>
    public class HMI_ButtonLinkCircleElem : HMI_ButtonElem
    {
        /// <summary>
        /// リンク
        /// </summary>
		[XmlAttribute("Href")]
        public string Href = "";
        /// <summary>
        /// 新しいタブで開く
        /// </summary>
		[XmlAttribute("OpenInNewPage")]
        public string OpenInNewPage = "0";
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_ButtonLinkCircleElem()
        {
            // 初期設定
            FontSize = "20px";
            Left = "0px";
            Top = "0px";
            Width = "100px";
            Height = "100px";
            BackColor = "silver";
            OpenInNewPage = "0";
        }

    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLinkCircleGreen_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkCircleGreen")]
        public List<HMI_ButtonLinkCircleElem>? HMI_ButtonLinkCircleGreens = null;
    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLinkCircleRed_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkCircleRed")]
        public List<HMI_ButtonLinkCircleElem>? HMI_ButtonLinkCircleReds = null;
    }

    /// <summary>
    /// xml情報(水色)
    /// </summary>
    public class HMI_ButtonLinkCircleRoyalblue_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkCircleRoyalblue")]
        public List<HMI_ButtonLinkCircleElem>? HMI_ButtonLinkCircleRoyalblues = null;
    }

    /// <summary>
    /// xml情報(オレンジ色)
    /// </summary>
    public class HMI_ButtonLinkCircleOrange_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkCircleOrange")]
        public List<HMI_ButtonLinkCircleElem>? HMI_ButtonLinkCircleOranges = null;
    }

    /// <summary>
    /// xml情報(青色)
    /// </summary>
    public class HMI_ButtonLinkCircleDarkblueOver_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkCircleDarkblueOver")]
        public List<HMI_ButtonLinkCircleElem>? HMI_ButtonLinkCircleDarkblueOvers = null;
    }

    /// <summary>
    /// ログ表示 xmlのロード
    /// </summary>
    public partial class HMI_LoadXml
    {
        /// <summary>
        /// Xmlファイルの読み込み 丸ボタン 緑
        /// </summary>
        /// <param name="hmiButtonLinkCircleGreenXml"></param>
        static public void Load_HMI_ButtonLinkCircleGreen_Xml(string folderName, ref HMI_ButtonLinkCircleGreen_Xml hmiButtonLinkCircleGreenXml)
        {
            hmiButtonLinkCircleGreenXml = LoadFromXml<HMI_ButtonLinkCircleGreen_Xml>($"{folderName}/HMI_ButtonLinkCircleGreen.xml");
            if (null != hmiButtonLinkCircleGreenXml)
            {
                if (null != hmiButtonLinkCircleGreenXml.HMI_ButtonLinkCircleGreens)
                {
                    foreach (var Item in hmiButtonLinkCircleGreenXml.HMI_ButtonLinkCircleGreens)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "green";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 丸ボタン 赤
        /// </summary>
        /// <param name="hmiButtonLinkCircleRedXml"></param>
        static public void Load_HMI_ButtonLinkCircleRed_Xml(string folderName, ref HMI_ButtonLinkCircleRed_Xml hmiButtonLinkCircleRedXml)
        {
            hmiButtonLinkCircleRedXml = LoadFromXml<HMI_ButtonLinkCircleRed_Xml>($"{folderName}/HMI_ButtonLinkCircleRed.xml");
            if (null != hmiButtonLinkCircleRedXml)
            {
                if (null != hmiButtonLinkCircleRedXml.HMI_ButtonLinkCircleReds)
                {
                    foreach (var Item in hmiButtonLinkCircleRedXml.HMI_ButtonLinkCircleReds)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "red";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 丸ボタン 薄青
        /// </summary>
        /// <param name="hmiButtonLinkCircleRoyalblueXml"></param>
        static public void Load_HMI_ButtonLinkCircleRoyalblue_Xml(string folderName, ref HMI_ButtonLinkCircleRoyalblue_Xml hmiButtonLinkCircleRoyalblueXml)
        {
            hmiButtonLinkCircleRoyalblueXml = LoadFromXml<HMI_ButtonLinkCircleRoyalblue_Xml>($"{folderName}/HMI_ButtonLinkCircleRoyalblue.xml");
            if (null != hmiButtonLinkCircleRoyalblueXml)
            {
                if (null != hmiButtonLinkCircleRoyalblueXml.HMI_ButtonLinkCircleRoyalblues)
                {
                    foreach (var Item in hmiButtonLinkCircleRoyalblueXml.HMI_ButtonLinkCircleRoyalblues)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "#008DDD";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 丸ボタン オレンジ
        /// </summary>
        /// <param name="hmiButtonLinkCircleOrangeXml"></param>
        static public void Load_HMI_ButtonLinkCircleOrange_Xml(string folderName, ref HMI_ButtonLinkCircleOrange_Xml hmiButtonLinkCircleOrangeXml)
        {
            hmiButtonLinkCircleOrangeXml = LoadFromXml<HMI_ButtonLinkCircleOrange_Xml>($"{folderName}/HMI_ButtonLinkCircleOrange.xml");
            if (null != hmiButtonLinkCircleOrangeXml)
            {
                if (null != hmiButtonLinkCircleOrangeXml.HMI_ButtonLinkCircleOranges)
                {
                    foreach (var Item in hmiButtonLinkCircleOrangeXml.HMI_ButtonLinkCircleOranges)
                    {
                        // 文字色の設定
                        Item.ForeColor = "White";
                        // 背景色の設定
                        Item.BackColor = "#FF6600";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 丸ボタン 濃青 マウスオーバー
        /// </summary>
        /// <param name="hmiButtonLinkCircleDarkblueOverXml"></param>
        static public void Load_HMI_ButtonLinkCircleDarkblueOver_Xml(string folderName, ref HMI_ButtonLinkCircleDarkblueOver_Xml hmiButtonLinkCircleDarkblueOverXml)
        {
            hmiButtonLinkCircleDarkblueOverXml = LoadFromXml<HMI_ButtonLinkCircleDarkblueOver_Xml>($"{folderName}/HMI_ButtonLinkCircleDarkblueOver.xml");
            if (null != hmiButtonLinkCircleDarkblueOverXml)
            {
                if (null != hmiButtonLinkCircleDarkblueOverXml.HMI_ButtonLinkCircleDarkblueOvers)
                {
                    foreach (var Item in hmiButtonLinkCircleDarkblueOverXml.HMI_ButtonLinkCircleDarkblueOvers)
                    {
                        // 文字色の設定
                        Item.ForeColor = "White";
                        // 背景色の設定
                        Item.BackColor = "#1b2538";
                        // 背景色の設定(マウスオーバー時)
                        Item.BackColorOver = "#A9BDE2";
                        // 高さの設定
                        Item.Height = Item.Width;
                    }
                }
            }
        }
    }
}
