﻿using System.Xml.Serialization;
using HMI_Base;

namespace HMI_Base
{
    /// <summary>
    /// 四角ボタン要素
    /// </summary>
    /// <remarks>
    /// 属性名はXmlAttributeで定義します。
    /// </remarks>
    public class HMI_ButtonLinkRectElem : HMI_ButtonElem
    {
        /// <summary>
        /// リンク
        /// </summary>
		[XmlAttribute("Href")]
        public string Href = "";
        /// <summary>
        /// 新しいタブで開く
        /// </summary>
		[XmlAttribute("OpenInNewPage")]
        public string OpenInNewPage = "0";
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_ButtonLinkRectElem()
        {
            // 初期設定
            FontSize = "20px";
            Left = "0px";
            Top = "0px";
            Width = "200px";
            Height = "100px";
            BackColor = "silver";
            OpenInNewPage = "0";
        }

    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLinkRectGreen_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkRectGreen")]
        public List<HMI_ButtonLinkRectElem>? HMI_ButtonLinkRectGreens = null;
    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonLinkRectRed_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkRectRed")]
        public List<HMI_ButtonLinkRectElem>? HMI_ButtonLinkRectReds = null;
    }

    /// <summary>
    /// xml情報(水色)
    /// </summary>
    public class HMI_ButtonLinkRectRoyalblue_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkRectRoyalblue")]
        public List<HMI_ButtonLinkRectElem>? HMI_ButtonLinkRectRoyalblues = null;
    }

    /// <summary>
    /// xml情報(オレンジ色)
    /// </summary>
    public class HMI_ButtonLinkRectOrange_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkRectOrange")]
        public List<HMI_ButtonLinkRectElem>? HMI_ButtonLinkRectOranges = null;
    }

    /// <summary>
    /// xml情報(青色)
    /// </summary>
    public class HMI_ButtonLinkRectDarkblueOver_Xml
    {
        [XmlArrayItem("HMI_ButtonLinkRectDarkblueOver")]
        public List<HMI_ButtonLinkRectElem>? HMI_ButtonLinkRectDarkblueOvers = null;
    }

    /// <summary>
    /// ログ表示 xmlのロード
    /// </summary>
    public partial class HMI_LoadXml
    {
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 緑
        /// </summary>
        /// <param name="hmiButtonLinkRectGreenXml"></param>
        static public void Load_HMI_ButtonLinkRectGreen_Xml(string folderName, ref HMI_ButtonLinkRectGreen_Xml hmiButtonLinkRectGreenXml)
        {
            hmiButtonLinkRectGreenXml = LoadFromXml<HMI_ButtonLinkRectGreen_Xml>($"{folderName}/HMI_ButtonLinkRectGreen.xml");
            if (null != hmiButtonLinkRectGreenXml)
            {
                if (null != hmiButtonLinkRectGreenXml.HMI_ButtonLinkRectGreens)
                {
                    foreach (var Item in hmiButtonLinkRectGreenXml.HMI_ButtonLinkRectGreens)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "green";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 赤
        /// </summary>
        /// <param name="hmiButtonLinkRectRedXml"></param>
        static public void Load_HMI_ButtonLinkRectRed_Xml(string folderName, ref HMI_ButtonLinkRectRed_Xml hmiButtonLinkRectRedXml)
        {
            hmiButtonLinkRectRedXml = LoadFromXml<HMI_ButtonLinkRectRed_Xml>($"{folderName}/HMI_ButtonLinkRectRed.xml");
            if (null != hmiButtonLinkRectRedXml)
            {
                if (null != hmiButtonLinkRectRedXml.HMI_ButtonLinkRectReds)
                {
                    foreach (var Item in hmiButtonLinkRectRedXml.HMI_ButtonLinkRectReds)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "red";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 薄青
        /// </summary>
        /// <param name="hmiButtonLinkRectRoyalblueXml"></param>
        static public void Load_HMI_ButtonLinkRectRoyalblue_Xml(string folderName, ref HMI_ButtonLinkRectRoyalblue_Xml hmiButtonLinkRectRoyalblueXml)
        {
            hmiButtonLinkRectRoyalblueXml = LoadFromXml<HMI_ButtonLinkRectRoyalblue_Xml>($"{folderName}/HMI_ButtonLinkRectRoyalblue.xml");
            if (null != hmiButtonLinkRectRoyalblueXml)
            {
                if (null != hmiButtonLinkRectRoyalblueXml.HMI_ButtonLinkRectRoyalblues)
                {
                    foreach (var Item in hmiButtonLinkRectRoyalblueXml.HMI_ButtonLinkRectRoyalblues)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "#008DDD";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン オレンジ
        /// </summary>
        /// <param name="hmiButtonLinkRectOrangeXml"></param>
        static public void Load_HMI_ButtonLinkRectOrange_Xml(string folderName, ref HMI_ButtonLinkRectOrange_Xml hmiButtonLinkRectOrangeXml)
        {
            hmiButtonLinkRectOrangeXml = LoadFromXml<HMI_ButtonLinkRectOrange_Xml>($"{folderName}/HMI_ButtonLinkRectOrange.xml");
            if (null != hmiButtonLinkRectOrangeXml)
            {
                if (null != hmiButtonLinkRectOrangeXml.HMI_ButtonLinkRectOranges)
                {
                    foreach (var Item in hmiButtonLinkRectOrangeXml.HMI_ButtonLinkRectOranges)
                    {
                        // 文字色の設定
                        Item.ForeColor = "White";
                        // 背景色の設定
                        Item.BackColor = "#FF6600";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 濃青 マウスオーバー
        /// </summary>
        /// <param name="hmiButtonLinkRectDarkblueOverXml"></param>
        static public void Load_HMI_ButtonLinkRectDarkblueOver_Xml(string folderName, ref HMI_ButtonLinkRectDarkblueOver_Xml hmiButtonLinkRectDarkblueOverXml)
        {
            hmiButtonLinkRectDarkblueOverXml = LoadFromXml<HMI_ButtonLinkRectDarkblueOver_Xml>($"{folderName}/HMI_ButtonLinkRectDarkblueOver.xml");
            if (null != hmiButtonLinkRectDarkblueOverXml)
            {
                if (null != hmiButtonLinkRectDarkblueOverXml.HMI_ButtonLinkRectDarkblueOvers)
                {
                    foreach (var Item in hmiButtonLinkRectDarkblueOverXml.HMI_ButtonLinkRectDarkblueOvers)
                    {
                        // 文字色の設定
                        Item.ForeColor = "White";
                        // 背景色の設定
                        Item.BackColor = "#1b2538";
                        // 背景色の設定(マウスオーバー時)
                        Item.BackColorOver = "#A9BDE2";
                    }
                }
            }
        }
    }
}
