﻿using System.Xml.Serialization;
using HMI_Base;

namespace HMI_Base
{
    /// <summary>
    /// 四角ボタン要素
    /// </summary>
    /// <remarks>
    /// 属性名はXmlAttributeで定義します。
    /// </remarks>
    public class HMI_ButtonRectElem : HMI_ButtonElem
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_ButtonRectElem()
        {
            // 初期設定
            FontSize = "20px";
            Left = "0px";
            Top = "0px";
            Width = "200px";
            Height = "100px";
            BackColor = "silver";
        }

    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonRectGreen_Xml
    {
        [XmlArrayItem("HMI_ButtonRectGreen")]
        public List<HMI_ButtonRectElem>? HMI_ButtonRectGreens = null;
    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_ButtonRectRed_Xml
    {
        [XmlArrayItem("HMI_ButtonRectRed")]
        public List<HMI_ButtonRectElem>? HMI_ButtonRectReds = null;
    }

    /// <summary>
    /// xml情報(水色)
    /// </summary>
    public class HMI_ButtonRectRoyalblue_Xml
    {
        [XmlArrayItem("HMI_ButtonRectRoyalblue")]
        public List<HMI_ButtonRectElem>? HMI_ButtonRectRoyalblues = null;
    }

    /// <summary>
    /// xml情報(オレンジ色)
    /// </summary>
    public class HMI_ButtonRectOrange_Xml
    {
        [XmlArrayItem("HMI_ButtonRectOrange")]
        public List<HMI_ButtonRectElem>? HMI_ButtonRectOranges = null;
    }

    /// <summary>
    /// xml情報(青色)
    /// </summary>
    public class HMI_ButtonRectDarkblueOver_Xml
    {
        [XmlArrayItem("HMI_ButtonRectDarkblueOver")]
        public List<HMI_ButtonRectElem>? HMI_ButtonRectDarkblueOvers = null;
    }

    /// <summary>
    /// ログ表示 xmlのロード
    /// </summary>
    public partial class HMI_LoadXml
    {
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 緑
        /// </summary>
        /// <param name="hmiButtonRectGreenXml"></param>
        static public void Load_HMI_ButtonRectGreen_Xml(string folderName, ref HMI_ButtonRectGreen_Xml hmiButtonRectGreenXml)
        {
            hmiButtonRectGreenXml = LoadFromXml<HMI_ButtonRectGreen_Xml>($"{folderName}/HMI_ButtonRectGreen.xml");
            if (null != hmiButtonRectGreenXml)
            {
                if (null != hmiButtonRectGreenXml.HMI_ButtonRectGreens)
                {
                    foreach (var Item in hmiButtonRectGreenXml.HMI_ButtonRectGreens)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "green";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 赤
        /// </summary>
        /// <param name="hmiButtonRectRedXml"></param>
        static public void Load_HMI_ButtonRectRed_Xml(string folderName, ref HMI_ButtonRectRed_Xml hmiButtonRectRedXml)
        {
            hmiButtonRectRedXml = LoadFromXml<HMI_ButtonRectRed_Xml>($"{folderName}/HMI_ButtonRectRed.xml");
            if (null != hmiButtonRectRedXml)
            {
                if (null != hmiButtonRectRedXml.HMI_ButtonRectReds)
                {
                    foreach (var Item in hmiButtonRectRedXml.HMI_ButtonRectReds)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "red";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 薄青
        /// </summary>
        /// <param name="hmiButtonRectRoyalblueXml"></param>
        static public void Load_HMI_ButtonRectRoyalblue_Xml(string folderName, ref HMI_ButtonRectRoyalblue_Xml hmiButtonRectRoyalblueXml)
        {
            hmiButtonRectRoyalblueXml = LoadFromXml<HMI_ButtonRectRoyalblue_Xml>($"{folderName}/HMI_ButtonRectRoyalblue.xml");
            if (null != hmiButtonRectRoyalblueXml)
            {
                if (null != hmiButtonRectRoyalblueXml.HMI_ButtonRectRoyalblues)
                {
                    foreach (var Item in hmiButtonRectRoyalblueXml.HMI_ButtonRectRoyalblues)
                    {
                        // 文字色の設定
                        Item.ForeColor = "black";
                        // 背景色の設定
                        Item.BackColor = "#008DDD";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン オレンジ
        /// </summary>
        /// <param name="hmiButtonRectOrangeXml"></param>
        static public void Load_HMI_ButtonRectOrange_Xml(string folderName, ref HMI_ButtonRectOrange_Xml hmiButtonRectOrangeXml)
        {
            hmiButtonRectOrangeXml = LoadFromXml<HMI_ButtonRectOrange_Xml>($"{folderName}/HMI_ButtonRectOrange.xml");
            if (null != hmiButtonRectOrangeXml)
            {
                if (null != hmiButtonRectOrangeXml.HMI_ButtonRectOranges)
                {
                    foreach (var Item in hmiButtonRectOrangeXml.HMI_ButtonRectOranges)
                    {
                        // 文字色の設定
                        Item.ForeColor = "White";
                        // 背景色の設定
                        Item.BackColor = "#FF6600";
                    }
                }
            }
        }
        /// <summary>
        /// Xmlファイルの読み込み 四角ボタン 濃青 マウスオーバー
        /// </summary>
        /// <param name="hmiButtonRectDarkblueOverXml"></param>
        static public void Load_HMI_ButtonRectDarkblueOver_Xml(string folderName, ref HMI_ButtonRectDarkblueOver_Xml hmiButtonRectDarkblueOverXml)
        {
            hmiButtonRectDarkblueOverXml = LoadFromXml<HMI_ButtonRectDarkblueOver_Xml>($"{folderName}/HMI_ButtonRectDarkblueOver.xml");
            if (null != hmiButtonRectDarkblueOverXml)
            {
                if (null != hmiButtonRectDarkblueOverXml.HMI_ButtonRectDarkblueOvers)
                {
                    foreach (var Item in hmiButtonRectDarkblueOverXml.HMI_ButtonRectDarkblueOvers)
                    {
                        // 文字色の設定
                        Item.ForeColor = "White";
                        // 背景色の設定
                        Item.BackColor = "#1b2538";
                        // 背景色の設定(マウスオーバー時)
                        Item.BackColorOver = "#A9BDE2";
                    }
                }
            }
        }
    }
}
