﻿using System.ComponentModel;
using System.Xml.Linq;
using System.Xml.Serialization;
using HMI_CommonLibrary;
using HMI_LogViewTable;
using static System.Net.Mime.MediaTypeNames;

#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif

namespace HMI_Base
{

    /// <summary>
    /// ログ表示要素
    /// </summary>
    /// <remarks>
    /// 属性名はXmlAttributeで定義します。
    /// </remarks>
    public class HMI_LogViewTableElem : HMI_LogElem
    {
        /// <summary>
        /// 文字色 ヘッダー
        /// </summary>
        [XmlAttribute("HeadForeColor")]
        public string HeadForeColor = "";
        /// <summary>
        /// 背景色 ヘッダー
        /// </summary>
        [XmlAttribute("HeadBackColor")]
        public string HeadBackColor = "";
        /// <summary>
        /// 文字色 値
        /// </summary>
        [XmlAttribute("BodyForeColor")]
        public string BodyForeColor = "";
        /// <summary>
        /// 背景色 値
        /// </summary>
        [XmlAttribute("BodyBackColor")]
        public string BodyBackColor = "";
        /// <summary>
        /// 背景色 ホバー時の色
        /// </summary>
        [XmlAttribute("HoverColor")]
        public string HoverColor = "";
        /// <summary>
        /// テーブルData
        /// </summary>
        [XmlElement("TableData")]
        public XmlTableData TableData { get; set; }
        /// <summary>
        /// チャートID
        /// </summary>
        [XmlAttribute("LogViewTableId")]
        public string LogViewTableId = "";
        /// <summary>
        /// 罫線の太さ
        /// </summary>
        [XmlAttribute("Border")]
        public string Border = "";
        /// <summary>
        /// 罫線の色
        /// </summary>
        [XmlAttribute("BorderColor")]
        public string BorderColor = "";

        //
        // クラス内で使用(xmlからは読み込みしない
        //
        [XmlIgnore]
        public List<string> TagNames { get; set; }
        [XmlIgnore]
        public Dictionary<string, HmiData> LogTagDataDic { get; set; }
        [XmlIgnore]
        public List<string> ColWidths { get; set; } = new List<string>();
        [XmlIgnore]
        public List<string> HeaderLabels { get; set; } = new List<string>();

        //public HeaderData HeadData { get; set; }
        [XmlIgnore]
        public List<HMI_LogViewTable.RowValue> RowValues { get; set; }


        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_LogViewTableElem()
        {
            // 初期設定
            FontSize = "1em";
            Tag = "";
            Text = "";
            FontSize = "1em";
            Left = "0px";
            Top = "0px";
            Width = "600px";
            Height = "100px";
            HeadForeColor = "#F0F0F0";
            HeadBackColor = "#1D3854";
            BodyForeColor = "#F0F0F0";
            BodyBackColor = "#2D5680";
            HoverColor = "#4e6e9b";
            TableData = new XmlTableData();
            TagNames = new List<string>();
            LogTagDataDic = new Dictionary<string, HmiData>();
            //HeadData = new HeaderData();
            ColWidths = new List<string>();
            HeaderLabels = new List<string>();
            RowValues = new List<HMI_LogViewTable.RowValue>();
        }

    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_LogViewTableBlue_Xml
    {
        [XmlArrayItem("HMI_LogViewTableBlue")]
        public List<HMI_LogViewTableElem> HMI_LogViewTableBlues = null;
    }

    /// <summary>
    /// xml情報
    /// </summary>
    public class HMI_LogViewTableWhite_Xml
    {
        [XmlArrayItem("HMI_LogViewTableWhite")]
        public List<HMI_LogViewTableElem> HMI_LogViewTableWhites = null;
    }

    /// <summary>
    /// ログ表示 xmlのロード
    /// </summary>
    public partial class HMI_LoadXml
    {
        /// <summary>
        /// 青色ログ表示 xmlのロード
        /// </summary>
        /// <param name="hmiLogViewTableBlueXml"></param>
        static public void Load_HMI_LogViewTableBlue_Xml(string folderName, ref HMI_LogViewTableBlue_Xml hmiLogViewTableBlueXml)
        {
            // Xmlファイルの読み込み
            hmiLogViewTableBlueXml = LoadFromXml<HMI_LogViewTableBlue_Xml>($"{folderName}/HMI_LogViewTableBlue.xml");
            if (null != hmiLogViewTableBlueXml)
            {
                if (null != hmiLogViewTableBlueXml.HMI_LogViewTableBlues)
                {
                    foreach (var Item in hmiLogViewTableBlueXml.HMI_LogViewTableBlues)
                    {
                        if (null != Item.LogTagDataDic && null != Item.TableData && null != Item.TableData.XmlColumns)
                        {
                            Item.LogTagDataDic.Clear(); //空にする
                            Item.TagNames.Clear();      //空にする
                            Item.HeaderLabels.Clear();  //空にする ヘッダ名
                            Item.ColWidths.Clear();  //空にする 列幅
                            Item.RowValues.Clear();
                            foreach (var ColumnItem in Item.TableData.XmlColumns)
                            {
                                if (null != ColumnItem.Header && "" != ColumnItem.Header)
                                {
                                    int count = 0;
                                    // edgeデータの取得用に作成
                                    foreach (string RowTagName in ColumnItem.XmlRows)
                                    {
                                        count++;
                                        if (Item.RowValues.Count < count)
                                            Item.RowValues.Add(new HMI_LogViewTable.RowValue());

                                        HmiData TagData = new HmiData() { TagName = RowTagName, TagValue = RowTagName, Digit = ColumnItem.Digit, IsTag = ColumnItem.IsTag };
                                        Item.RowValues[count - 1].ColValues.Add(TagData);

                                        //タグの時RT-edgeから読込対象とする
                                        if (ColumnItem.IsTag == true)
                                        {
                                            Item.TagNames.Add(RowTagName);
                                            if (!Item.LogTagDataDic.ContainsKey(RowTagName))
                                                Item.LogTagDataDic.Add(RowTagName, TagData);
                                        }
                                    }
                                    Item.ColWidths.Add(ColumnItem.ColWidth);
                                    Item.HeaderLabels.Add(ColumnItem.Header);
                                }
                            }
                        }
                        // IDの設定
                        Item.LogViewTableId = "LogViewTableBlue_" + Item.LogViewTableId;
                        // 罫線　太さの設定
                        Item.BorderColor = "1";
                        // 罫線　色の設定
                        Item.BorderColor = "#7F8591";
                    }
                }
            }
        }
        /// <summary>
        /// 白 ログテーブルのxmlの読み込み
        /// </summary>
        /// <param name="hmiLogViewTableWhiteXml"></param>
        static public void Load_HMI_LogViewTableWhite_Xml(string folderName, ref HMI_LogViewTableWhite_Xml hmiLogViewTableWhiteXml)
        {
            hmiLogViewTableWhiteXml = LoadFromXml<HMI_LogViewTableWhite_Xml>($"{folderName}/HMI_LogViewTableWhite.xml");
            if (null != hmiLogViewTableWhiteXml)
            {
                if (null != hmiLogViewTableWhiteXml.HMI_LogViewTableWhites)
                {
                    foreach (var Item in hmiLogViewTableWhiteXml.HMI_LogViewTableWhites)
                    {
                        if (null != Item.LogTagDataDic && null != Item.TableData && null != Item.TableData.XmlColumns)
                        {
                            Item.LogTagDataDic.Clear(); //空にする
                            Item.TagNames.Clear();      //空にする
                            Item.HeaderLabels.Clear();  //空にする ヘッダ名
                            Item.ColWidths.Clear();  //空にする 列幅
                            Item.RowValues.Clear();
                            foreach (var ColumnItem in Item.TableData.XmlColumns)
                            {
                                if (null != ColumnItem.Header && "" != ColumnItem.Header)
                                {
                                    int count = 0;
                                    // edgeデータの取得用に作成
                                    foreach (string RowTagName in ColumnItem.XmlRows)
                                    {
                                        count++;
                                        if (Item.RowValues.Count < count)
                                            Item.RowValues.Add(new HMI_LogViewTable.RowValue());

                                        HmiData TagData = new HmiData() { TagName = RowTagName, TagValue = RowTagName, Digit = ColumnItem.Digit, IsTag = ColumnItem.IsTag };
                                        Item.RowValues[count - 1].ColValues.Add(TagData);

                                        //タグの時RT-edgeから読込対象とする
                                        if (ColumnItem.IsTag == true)
                                        {
                                            Item.TagNames.Add(RowTagName);
                                            if (!Item.LogTagDataDic.ContainsKey(RowTagName))
                                                Item.LogTagDataDic.Add(RowTagName, TagData);
                                        }
                                    }
                                    Item.ColWidths.Add(ColumnItem.ColWidth);
                                    Item.HeaderLabels.Add(ColumnItem.Header);
                                }
                            }
                        }
                        // 文字色 ヘッダーの設定
                        Item.HeadForeColor = "black";
                        // 背景色 ヘッダーの設定
                        Item.HeadBackColor = "#BFBFBF";
                        // 文字色 値の設定
                        Item.BodyForeColor = "black";
                        // 背景色 値の設定
                        Item.BodyBackColor = "white";
                        // 背景色　ホバー時の設定
                        Item.HoverColor = "white";
                        // 罫線　太さの設定
                        Item.BorderColor = "1";
                        // 罫線　色の設定
                        Item.BorderColor = "#7F8591";
                        // IDの設定
                        Item.LogViewTableId = "TableWhite_" + Item.LogViewTableId;
                    }
                }
            }
        }
    }
}
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif
