﻿using System.ComponentModel;
using System.Xml.Linq;
using System.Xml.Serialization;
using HMI_CommonLibrary;
using HMI_LogViewTable;
using static System.Net.Mime.MediaTypeNames;

#nullable disable

namespace HMI_Base
{

    /// <summary>
    /// ログ表示要素
    /// </summary>
    /// <remarks>
    /// 属性名はXmlAttributeで定義します。
    /// </remarks>
    public class HMI_TextBoxSingleElem : HMI_TextBoxElem
    {
        /// <summary>
        /// データの型
        /// </summary>
        [XmlAttribute("InputDataType")]
        public int InputDataType = 0;
        /// <summary>
        /// タイルの表示幅(TileInputWidth)
        /// </summary>
        [XmlAttribute("InputWidth")]
        public string InputWidth = "";
        /// <summary>
        /// タイルの表示幅(TileButtonWidth)
        /// </summary>
        [XmlAttribute("ButtonWidth")]
        public string ButtonWidth = "";
        /// <summary>
        /// 背景色
        /// </summary>
        [XmlAttribute("InputBackColor")]
        public string InputBackColor = "";
        /// <summary>
        /// 文字色
        /// </summary>
		[XmlAttribute("InputForeColor")]
        public string InputForeColor = "";
        /// <summary>
        /// 背景色
        /// </summary>
        [XmlAttribute("ButtonBackColor")]
        public string ButtonBackColor = "";
        /// <summary>
        /// 文字色
        /// </summary>
		[XmlAttribute("ButtonForeColor")]
        public string ButtonForeColor = "";
        /// <summary>
        /// ComponentType
        /// </summary>
        [XmlAttribute("ButtonType")]
        public int ButtonType = 3;
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_TextBoxSingleElem()
        {
            // 初期設定
            FontSize = "20px";
            Left = "0px";
            Top = "0px";
            InputWidth = "200px";
            ButtonWidth = "200px";
            Height = "100px";
            InputBackColor = "white";
            InputForeColor = "black";
            ButtonBackColor = "blue";
            InputForeColor = "white";
            ButtonType = 3;
        }

    }

    /// <summary>
    /// xml情報
    /// </summary>
	public class HMI_TextBoxSingleWhiteBlue_Xml
    {
        [XmlArrayItem("HMI_TextBoxSingleWhiteBlue")]
        public List<HMI_TextBoxSingleElem> HMI_TextBoxSingleWhiteBlues = null;
    }
    public class HMI_TextBoxSingleWhiteGreen_Xml
    {
        [XmlArrayItem("HMI_TextBoxSingleWhiteGreen")]
        public List<HMI_TextBoxSingleElem> HMI_TextBoxSingleWhiteGreens = null;
    }
    public class HMI_TextBoxSingleWhiteOrange_Xml
    {
        [XmlArrayItem("HMI_TextBoxSingleWhiteOrange")]
        public List<HMI_TextBoxSingleElem> HMI_TextBoxSingleWhiteOranges = null;
    }

    /// <summary>
    /// ログ表示 xmlのロード
    /// </summary>
    public partial class HMI_LoadXml
    {
        /// <summary>
        /// 青色ログ表示 xmlのロード
        /// </summary>
        /// <param name="hmiTextBoxSingleWhiteBlueXml"></param>
        static public void Load_HMI_TextBoxSingleWhiteBlue_Xml(string folderName, ref HMI_TextBoxSingleWhiteBlue_Xml hmiTextBoxSingleWhiteBlueXml)
        {
            // Xmlファイルの読み込み
            hmiTextBoxSingleWhiteBlueXml = LoadFromXml<HMI_TextBoxSingleWhiteBlue_Xml>($"{folderName}/HMI_TextBoxSingleWhiteBlue.xml");
            if (null != hmiTextBoxSingleWhiteBlueXml)
            {
                if (null != hmiTextBoxSingleWhiteBlueXml.HMI_TextBoxSingleWhiteBlues)
                {
                    foreach (var Item in hmiTextBoxSingleWhiteBlueXml.HMI_TextBoxSingleWhiteBlues)
                    {
                        // 文字色の設定
                        Item.InputForeColor = "black";
                        Item.ButtonForeColor = "white";
                        // 背景色の設定
                        Item.InputBackColor = "white";
                        Item.ButtonBackColor = "blue";
                    }
                }
            }
        }
        /// <summary>
        /// 緑色ログ表示 xmlのロード
        /// </summary>
        /// <param name="hmiTextBoxSingleWhiteGreenXml"></param>
        static public void Load_HMI_TextBoxSingleWhiteGreen_Xml(string folderName, ref HMI_TextBoxSingleWhiteGreen_Xml hmiTextBoxSingleWhiteGreenXml)
        {
            // Xmlファイルの読み込み
            hmiTextBoxSingleWhiteGreenXml = LoadFromXml<HMI_TextBoxSingleWhiteGreen_Xml>($"{folderName}/HMI_TextBoxSingleWhiteGreen.xml");
            if (null != hmiTextBoxSingleWhiteGreenXml)
            {
                if (null != hmiTextBoxSingleWhiteGreenXml.HMI_TextBoxSingleWhiteGreens)
                {
                    foreach (var Item in hmiTextBoxSingleWhiteGreenXml.HMI_TextBoxSingleWhiteGreens)
                    {
                        // 文字色の設定
                        Item.InputForeColor = "black";
                        Item.ButtonForeColor = "white";
                        // 背景色の設定
                        Item.InputBackColor = "white";
                        Item.ButtonBackColor = "green";
                    }
                }
            }
        }
        /// <summary>
        /// オレンジ色ログ表示 xmlのロード
        /// </summary>
        /// <param name="hmiTextBoxSingleWhiteOrangeXml"></param>
        static public void Load_HMI_TextBoxSingleWhiteOrange_Xml(string folderName, ref HMI_TextBoxSingleWhiteOrange_Xml hmiTextBoxSingleWhiteOrangeXml)
        {
            // Xmlファイルの読み込み
            hmiTextBoxSingleWhiteOrangeXml = LoadFromXml<HMI_TextBoxSingleWhiteOrange_Xml>($"{folderName}/HMI_TextBoxSingleWhiteOrange.xml");
            if (null != hmiTextBoxSingleWhiteOrangeXml)
            {
                if (null != hmiTextBoxSingleWhiteOrangeXml.HMI_TextBoxSingleWhiteOranges)
                {
                    foreach (var Item in hmiTextBoxSingleWhiteOrangeXml.HMI_TextBoxSingleWhiteOranges)
                    {
                        // 文字色の設定
                        Item.InputForeColor = "black";
                        Item.ButtonForeColor = "white";
                        // 背景色の設定
                        Item.InputBackColor = "white";
                        Item.ButtonBackColor = "orange";
                    }
                }
            }
        }
    }

}
#nullable restore

