﻿using System;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Text;

namespace HMI_Base
{
    public class MnXmlUtility
    {
        /// <summary>
        /// XMLファイルからシリアライズ可能なオブジェクトを読み込みます。
        /// </summary>
        /// <typeparam name="T">読み込むオブジェクトの型</typeparam>
        /// <param name="filePath">オブジェクトが保存されたXMLファイルパス
        /// <returns>デシリアライズされたオブジェクト</returns>
        public static T? LoadFromFile<T>(string filePath)
            where T : class
        {
            T? obj = null;
            var serializer = new XmlSerializer(typeof(T));

            
            try
            {
                if (!File.Exists(filePath))// ファイルが存在するときだけ実行可能
                {
                    throw new FileNotFoundException();
                }
            }
            catch (FileNotFoundException)
            {
                return null;
            }

            // ファイルからオブジェクトの読み込み（デシリアライズ）
            try
            {
                using (var fs = new FileStream(filePath, FileMode.Open, FileAccess.Read))
                {
                    obj = serializer.Deserialize(fs) as T;
                }
            }
            catch (Exception)
            {
                return null;
            }

            return obj;
        }

        /// <summary>
        /// シリアライズ可能なオブジェクトをXMLファイルへ書き込みます。
        /// </summary>
        /// <param name="filePath">書き込むXMLファイルパス
        /// <param name="obj">シリアライズするオブジェクト
        /// <param name="type">シリアライズするオブジェクトの型
        public static void SaveToFile(string filePath, object obj, Type type)
        {
            try
            {
                var xmlSerializer = new XmlSerializer(type);
                using (var streamwriter = new StreamWriter(filePath, false, Encoding.UTF8)) //書き込み処理
                {
                    xmlSerializer.Serialize(streamwriter, obj);
                    streamwriter.Flush();
                }
            }
            catch (Exception)
            {
                throw;
            }
        }
    }
}
