﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using Microsoft.AspNetCore.Components.Authorization;
using System.Security.Claims;
using System.Threading.Tasks;

public class CustomAuthenticationStateProvider : AuthenticationStateProvider
{
    // 現在のユーザー情報を保持する ClaimsPrincipal。初期状態では空の ClaimsIdentity
    private ClaimsPrincipal _currentUser = new ClaimsPrincipal(new ClaimsIdentity());

    // 認証状態を非同期で取得するメソッド
    public override Task<AuthenticationState> GetAuthenticationStateAsync()
    {
        // 現在のユーザー情報を使って AuthenticationState を返す
        return Task.FromResult(new AuthenticationState(_currentUser));
    }

    // ユーザーを認証済みとしてマークし、状態を更新するメソッド
    public Task MarkUserAsAuthenticated(ClaimsPrincipal user)
    {
        // 現在のユーザー情報を更新
        _currentUser = user;

        // 認証状態が変わったことを通知（UI側で認証状態の変化を反映するため）
        NotifyAuthenticationStateChanged(GetAuthenticationStateAsync());

        // 完了したタスクを返す（非同期で処理が終了したことを通知）
        return Task.CompletedTask;
    }

    // ユーザーをログアウトさせ、認証状態を更新するメソッド
    public Task MarkUserAsLoggedOut()
    {
        // 空の ClaimsIdentity に切り替えてログアウト状態にする
        _currentUser = new ClaimsPrincipal(new ClaimsIdentity());

        // ログアウトしたことを通知（UI側で認証状態の変化を反映するため）
        NotifyAuthenticationStateChanged(GetAuthenticationStateAsync());

        // 完了したタスクを返す（非同期で処理が終了したことを通知）
        return Task.CompletedTask;
    }
}
