﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.Json;
using System.Text.Json.Serialization;

// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が無効になる(従来通り)
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif
namespace HMI_Container
{
    /// <summary>
    /// メッセージ管理クラス
    /// </summary>
    public class HMI_MngMsg
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_MngMsg()
        {
            // 要求伝文(ヘッダ部)の初期化
            m_clReqHead = new HMI_MsgHead();
            // 応答伝文(ヘッダ部)の初期化
            m_clResHead = new HMI_MsgHead();
            // 要求伝文(ヘッダ部)ACKの初期化
            m_clReqHeadAck = new HMI_MsgAck();
            // 要求伝文(データ部)ACKの初期化
            m_clReqDataAck = new HMI_MsgAck();
            // 応答伝文(ヘッダ部)ACKの初期化
            m_clResHeadAck = new HMI_MsgAck();
            // 応答伝文(データ部)ACKの初期化
            m_clResDataAck = new HMI_MsgAck();
        }

        /// <summary>
        /// 要求伝文(ヘッダ部)
        /// </summary>
        public HMI_MsgHead m_clReqHead;

        /// <summary>
        /// 応答伝文(ヘッダ部)
        /// </summary>
        public HMI_MsgHead m_clResHead;

        /// <summary>
        /// 要求伝文(ヘッダ部)ACK
        /// </summary>
        public HMI_MsgAck m_clReqHeadAck;

        /// <summary>
        /// 応答伝文(ヘッダ部)ACK
        /// </summary>
        public HMI_MsgAck m_clResHeadAck;

        /// <summary>
        /// 要求伝文(データ部)ACK
        /// </summary>
        public HMI_MsgAck m_clReqDataAck;

        /// <summary>
        /// 応答伝文(データ部)ACK
        /// </summary>
        public HMI_MsgAck m_clResDataAck;

    }
    /// <summary>
    /// 要求/応答伝文(データ部)：タグ情報
    /// </summary>
    [JsonConverter(typeof(MsgDataTagValue.MsgDataTagValueConverter))]
    public class MsgDataTagValue
    {
        /// <summary>
        /// タグ名
        /// </summary>
        public string m_srName { get; set; }
        /// <summary>
        /// 型
        /// </summary>
        public UInt16 m_woType { get; set; }
        /// <summary>
        /// 値
        /// </summary>
        public object m_obVal { get; set; }

        public static object ConvertValue(UInt16 woType, object value)
        {
            try
            {
                switch (woType)
                {
                    case (ushort)egTagDataType.Boolean:   // bool
                        return Convert.ToBoolean(value);
                    case (ushort)egTagDataType.SByte:   // SByte
                        return Convert.ToSByte(value);
                    case (ushort)egTagDataType.Byte:   // Byte
                        return Convert.ToByte(value);
                    case (ushort)egTagDataType.Int16:   // Int16
                        return Convert.ToInt16(value);
                    case (ushort)egTagDataType.UInt16:   // UInt16
                        return Convert.ToUInt16(value);
                    case (ushort)egTagDataType.Int32:   // Int32
                        return Convert.ToInt32(value);
                    case (ushort)egTagDataType.UInt32:   // UInt32
                        return Convert.ToUInt32(value);
                    case (ushort)egTagDataType.Int64:   // Int64
                        return Convert.ToInt64(value);
                    case (ushort)egTagDataType.UInt64:   // UInt64
                        return Convert.ToUInt64(value);
                    case (ushort)egTagDataType.Float:   // Float
                        return Convert.ToSingle(value);
                    case (ushort)egTagDataType.Double:   // Double
                        return Convert.ToDouble(value);
                    case (ushort)egTagDataType.String:   // String
                        return Convert.ToString(value);
                    default:
                        throw new InvalidOperationException($"サポートされていないm_woType: {woType}");
                }
            }
            catch
            {
                return 0;
            }
        }
        // m_woTypeに基づいてm_obValを適切な型に変換
        public object _ConvertValue(object value)
        {
            return ConvertValue(m_woType, value);
        }
        /// <summary>
        /// 型に対応したオブジェクトを返す
        /// </summary>
        /// <returns></returns>
        public object GetConvertObjValue()
        {
            return _ConvertValue(m_obVal);
        }

        public class MsgDataTagValueConverter : JsonConverter<MsgDataTagValue>
        {
            public override MsgDataTagValue Read(ref Utf8JsonReader reader, Type typeToConvert, JsonSerializerOptions options)
            {
                var result = new MsgDataTagValue();

                if (reader.TokenType != JsonTokenType.StartObject)
                    throw new JsonException("開始オブジェクトトークンが必要です。");

                while (reader.Read())
                {
                    if (reader.TokenType == JsonTokenType.EndObject)
                        return result;

                    if (reader.TokenType != JsonTokenType.PropertyName)
                        throw new JsonException("プロパティ名が必要です。");

                    string propertyName = reader.GetString();
                    reader.Read();

                    switch (propertyName)
                    {
                        case "m_srName":
                            result.m_srName = reader.GetString();
                            break;

                        case "m_woType":
                            result.m_woType = reader.GetUInt16();
                            break;

                        case "m_obVal":
                            result.m_obVal = ReadObjectValue(result.m_woType, ref reader);
                            break;

                        default:
                            throw new JsonException($"未知のプロパティ名: {propertyName}");
                    }
                }

                throw new JsonException("終了オブジェクトトークンが必要です。");
            }

            public override void Write(Utf8JsonWriter writer, MsgDataTagValue value, JsonSerializerOptions options)
            {
                writer.WriteStartObject();

                writer.WritePropertyName("m_srName");
                writer.WriteStringValue(value.m_srName);

                writer.WritePropertyName("m_woType");
                writer.WriteNumberValue(value.m_woType);

                writer.WritePropertyName("m_obVal");
                WriteObjectValue(writer, value.m_obVal);

                writer.WriteEndObject();
            }

            private object ReadObjectValue(ushort woType, ref Utf8JsonReader reader)
            {
                switch (reader.TokenType)
                {
                    case JsonTokenType.True:
                        return true;
                    case JsonTokenType.False:
                        return false;
                    case JsonTokenType.Number:
                        if (woType == (ushort)egTagDataType.Boolean && reader.TryGetByte(out byte byteBoolValue))
                        {
                            //ここにはこないかも 0の場合はfalse
                            if (byteBoolValue == 0)
                                return false;
                            return true;
                        }
                        if (woType == (ushort)egTagDataType.Byte && reader.TryGetByte(out byte byteValue))
                            return byteValue;
                        if (woType == (ushort)egTagDataType.SByte && reader.TryGetSByte(out sbyte sbyteValue))
                            return sbyteValue;
                        if (woType == (ushort)egTagDataType.Int16 && reader.TryGetInt16(out short int16Value))
                            return int16Value;
                        if (woType == (ushort)egTagDataType.UInt16 && reader.TryGetUInt16(out ushort uint16Value))
                            return uint16Value;
                        if (woType == (ushort)egTagDataType.Int32 && reader.TryGetInt32(out int int32Value))
                            return int32Value;
                        if (woType == (ushort)egTagDataType.UInt32 && reader.TryGetUInt32(out uint uint32Value))
                            return uint32Value;
                        if (woType == (ushort)egTagDataType.Int64 && reader.TryGetInt64(out long int64Value))
                            return int64Value;
                        if (woType == (ushort)egTagDataType.UInt64 && reader.TryGetUInt64(out ulong uint64Value))
                            return uint64Value;
                        if (woType == (ushort)egTagDataType.Float && reader.TryGetSingle(out float floatValue))
                            return floatValue;
                        if (woType == (ushort)egTagDataType.Double && reader.TryGetDouble(out double doubleValue))
                            return doubleValue;

                        return reader.GetDouble();  // とりあえずすべて数値として読み込み
  
                    case JsonTokenType.String:
                        return reader.GetString();
                    default:
                        throw new JsonException("サポートされていない型です");
                }
            }

            private void WriteObjectValue(Utf8JsonWriter writer, object value)
            {
                if (value is bool boolValue)
                {
                    writer.WriteBooleanValue(boolValue);
                }
                else if (value is byte byteValue)
                {
                    writer.WriteNumberValue(byteValue);
                }
                else if (value is sbyte sbyteValue)
                {
                    writer.WriteNumberValue(sbyteValue);
                }
                else if (value is short int16Value)
                {
                    writer.WriteNumberValue(int16Value);
                }
                else if (value is ushort uint16Value)
                {
                    writer.WriteNumberValue(uint16Value);
                }
                else if (value is int int32Value)
                {
                    writer.WriteNumberValue(int32Value);
                }
                else if (value is uint uint32Value)
                {
                    writer.WriteNumberValue(uint32Value);
                }
                else if (value is long int64Value)
                {
                    writer.WriteNumberValue(int64Value);
                }
                else if (value is ulong uint64Value)
                {
                    writer.WriteNumberValue(uint64Value);
                }
                else if (value is float floatValue)
                {
                    writer.WriteNumberValue(floatValue);
                }
                else if (value is double doubleValue)
                {
                    writer.WriteNumberValue(doubleValue);
                }
                else if (value is string stringValue)
                {
                    writer.WriteStringValue(stringValue);
                }
                else
                {
                    throw new JsonException("サポートされていない型です");
                }
            }
        }
    }
}
// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が有効になる
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif