﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using System.Text.Json.Serialization;

// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が無効になる(従来通り)
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif
namespace HMI_Container
{
    /// <summary>
    /// タグ読み込み管理クラス
    /// </summary>
    /// <remarks>
    /// クライアントとサーバー間とのタグ読み込みで使用するクラスです。
    /// </remarks>
    public class HMI_MngMsgReadTag
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_MngMsgReadTag()
        {
            // 要求伝文(データ部)初期化
            InitReqData();
            // 応答伝文(データ部)初期化
            InitResData();
        }

        /// <summary>
        /// デストラクタ
        /// </summary>
        ~HMI_MngMsgReadTag()
        {
        }

        /// <summary>
        /// 要求伝文(データ部)
        /// </summary>
        public class MsgReqDataReadTags
        {
            /// <summary>
            /// MsgHead
            /// </summary>
            public MsgHead m_Head { get; set; }

            /// <summary>
            /// タグ情報
            /// </summary>
            public List<MsgDataTagName> m_lsTags { get; set; }
        }

        /// <summary>
        /// 要求伝文(データ部)：タグ情報
        /// </summary>
        public class MsgDataTagName
        {
            /// <summary>
            /// タグ名
            /// </summary>
            public string m_srName { get; set; }
        }

        /// <summary>
        /// 要求伝文(データ部)
        /// </summary>
        public MsgReqDataReadTags m_clReqData;

        /// <summary>
        /// シリアライズされた要求伝文(データ部)
        /// </summary>
        public byte[] m_byReqData;

        /// <summary>
        /// 要求伝文(データ部)初期化
        /// </summary>
        private void InitReqData()
        {
            // 要求伝文(データ部)
            m_clReqData = new MsgReqDataReadTags();
            m_clReqData.m_Head = new MsgHead();
            m_clReqData.m_Head.m_dwPrefix = (UInt32)HMI_Msg.MSG_PREFIX;
            m_clReqData.m_Head.m_woMsgNo = (UInt16)HMI_Msg.MsgNo.ReqReadTag;
            m_clReqData.m_lsTags = new List<MsgDataTagName>();
            m_byReqData = null;
        }

        /// <summary>
        /// 要求伝文(データ部)を確保する
        /// </summary>
        /// <param name="dwDataSize"></param>
        public void AllocReqData(UInt32 dwDataSize)
        {
            m_byReqData = new byte[dwDataSize];
        }

        /// <summary>
        /// 要求伝文(データ部)をシリアライズする
        /// </summary>
        public void SerializeReqData()
        {
            var options = new JsonSerializerOptions
            {
                Converters = { new MsgDataTagValue.MsgDataTagValueConverter() }
            };
            m_byReqData = JsonSerializer.SerializeToUtf8Bytes(m_clReqData, options);
        }

        /// <summary>
        /// 要求伝文(データ部)をデシリアライズする
        /// </summary>
        public void DeserializeReqData()
        {
            // バイト配列から MsgResDataReadTags をデシリアライズ
            var options = new JsonSerializerOptions
            {
                Converters = { new MsgDataTagValue.MsgDataTagValueConverter() }
            };
            m_clReqData = JsonSerializer.Deserialize<MsgReqDataReadTags>(m_byReqData, options);
        }

        /// <summary>
        /// 要求伝文(データ部)：タグ情報追加
        /// </summary>
        /// <param name="srName">タグ名</param>
        public void AddReqDataTag(string srName)
        {
            var clItem = new MsgDataTagName();
            clItem.m_srName = srName;
            m_clReqData.m_lsTags.Add(clItem);
        }
        /// <summary>
        /// タグをクリアする
        /// </summary>
        public void ClearReqDataTags()
        {
            m_clReqData.m_lsTags.Clear();
        }

        /// <summary>
        /// 応答伝文(データ部)
        /// </summary>
        public class MsgResDataReadTags
        {
            /// <summary>
            /// MsgHead
            /// </summary>
            public MsgHead m_Head { get; set; }

            /// <summary>
            /// メッセージNo
            /// </summary>
            public UInt16 m_woMsgNo { get; set; }

            /// <summary>
            /// タグ情報
            /// </summary>
            public List<MsgDataTagValue> m_lsTags { get; set; }
        }

        /// <summary>
        /// 応答伝文(データ部)
        /// </summary>
        public MsgResDataReadTags m_clResData;

        /// <summary>
        /// シリアライズされた応答伝文(データ部)
        /// </summary>
        public byte[] m_byResData;

        /// <summary>
        /// 応答伝文(データ部)初期化
        /// </summary>
        private void InitResData()
        {
            // 応答伝文(データ部)
            m_clResData = new MsgResDataReadTags();
            m_clResData.m_Head = new MsgHead();
            m_clResData.m_Head.m_dwPrefix = (UInt32)HMI_Msg.MSG_PREFIX;
            m_clResData.m_Head.m_woMsgNo = (UInt16)HMI_Msg.MsgNo.ResReadTag;
            m_clResData.m_lsTags = new List<MsgDataTagValue>();
            m_byResData = null;
        }

        /// <summary>
        /// 応答伝文(データ部)をシリアライズする
        /// </summary>
        public void SerializeResData()
        {
            var options = new JsonSerializerOptions
            {
                Converters = { new MsgDataTagValue.MsgDataTagValueConverter() },
            };
            m_byResData = JsonSerializer.SerializeToUtf8Bytes(m_clResData, options);
        }
        /// <summary>
        /// 応答伝文(データ部)をデシリアライズする
        /// </summary>
        public void DeserializeResData()
        {
            // バイト配列から MsgResDataReadTags をデシリアライズ
            var options = new JsonSerializerOptions
            {
                Converters = { new MsgDataTagValue.MsgDataTagValueConverter() }
            };
            string jsonString = System.Text.Encoding.UTF8.GetString(m_byResData);
            m_clResData = JsonSerializer.Deserialize<MsgResDataReadTags>(jsonString, options);
        }
    }
}
// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が有効になる
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif
