﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using System.Text.Json.Serialization;

// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が無効になる(従来通り)
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif
namespace HMI_Container
{
    /// <summary>
    /// タグ読み込み管理クラス
    /// </summary>
    /// <remarks>
    /// クライアントとサーバー間とのタグ読み込みで使用するクラスです。
    /// </remarks>
    public class HMI_MngMsgWriteTag
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_MngMsgWriteTag()
        {
            // 要求伝文(データ部)初期化
            InitReqData();
            // 応答伝文(データ部)なし
        }

        /// <summary>
        /// デストラクタ
        /// </summary>
        ~HMI_MngMsgWriteTag()
        {
        }

        /// <summary>
        /// 要求伝文(データ部)
        /// </summary>
        public class MsgReqDataWriteTags
        {
            /// <summary>
            /// MsgHead
            /// </summary>
            public MsgHead m_Head { get; set; }

            /// <summary>
            /// タグ情報
            /// </summary>
            public List<MsgDataTagValue> m_lsTags { get; set; }
        }

        

        /// <summary>
        /// 要求伝文(データ部)
        /// </summary>
        public MsgReqDataWriteTags m_clReqData;

        /// <summary>
        /// シリアライズされた要求伝文(データ部)
        /// </summary>
        public byte[] m_byReqData;

        /// <summary>
        /// 要求伝文(データ部)初期化
        /// </summary>
        private void InitReqData()
        {
            // 要求伝文(データ部)
            m_clReqData = new MsgReqDataWriteTags();
            m_clReqData.m_Head = new MsgHead();
            m_clReqData.m_Head.m_dwPrefix = (UInt32)HMI_Msg.MSG_PREFIX;
            m_clReqData.m_Head.m_woMsgNo = (UInt16)HMI_Msg.MsgNo.ReqReadTag;
            m_clReqData.m_lsTags = new List<MsgDataTagValue>();
            m_byReqData = null;
        }

        /// <summary>
        /// 要求伝文(データ部)を確保する
        /// </summary>
        /// <param name="dwDataSize"></param>
        public void AllocReqData(UInt32 dwDataSize)
        {
            m_byReqData = new byte[dwDataSize];
        }

        /// <summary>
        /// 要求伝文(データ部)をシリアライズする
        /// </summary>
        public void SerializeReqData()
        {
            var options = new JsonSerializerOptions
            {
                Encoder = System.Text.Encodings.Web.JavaScriptEncoder.UnsafeRelaxedJsonEscaping
            };
            m_byReqData = JsonSerializer.SerializeToUtf8Bytes(m_clReqData, options);
        }

        ///
        public void DeserializeReqData()
        {
            m_clReqData = JsonSerializer.Deserialize<MsgReqDataWriteTags>(m_byReqData);
        }

        /// <summary>
        /// 要求伝文(データ部)：タグ情報追加
        /// </summary>
        /// <param name="srName">タグ名</param>
        public void AddReqDataTag(string srName, UInt16 woType, object val )
        {
            var clItem = new MsgDataTagValue();
            clItem.m_srName = srName;
            clItem.m_woType = woType;
            clItem.m_obVal = val;
            m_clReqData.m_lsTags.Add(clItem);
        }
        /// <summary>
        /// タグの削除
        /// </summary>
        public void ClearReqDataTag()
        {
            m_clReqData.m_lsTags.Clear();
        }
    }
}
// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が有効になる
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif
