﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;

// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が無効になる(従来通り)
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif
namespace HMI_Container
{
    /// <summary>
    /// メッセージクラス
    /// </summary>
    public class HMI_Msg
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_Msg()
        {
        }

        /// <summary>
        /// Prefix
        /// </summary>
        public const UInt32 MSG_PREFIX = 0x484D4931;

        /// <summary>
        /// メッセージNo
        /// </summary>
        public enum MsgNo : UInt16
        {
            /// <summary>
            /// タグデータ読み込み要求
            /// </summary>
            ReqReadTag = 0x8001,
            /// <summary>
            /// タグデータ読み込み応答
            /// </summary>
            ResReadTag = 0x9001,

            /// <summary>
            /// タグデータ書き込み要求
            /// </summary>
            ReqWriteTag = 0x8011,
            /// <summary>
            /// タグデータ書き込み応答
            /// </summary>
            ResWriteTag = 0x9011,
        }

        /// <summary>
        /// エラー
        /// </summary>
        public enum ErrCode : UInt32
        {
            /// <summary>
            /// 正常終了
            /// </summary>
            Ok = 0,
            /// <summary>
            /// プリフィックス不一致
            /// </summary>
            UnmacthPrefix,
            /// <summary>
            /// メッセージNo不一致
            /// </summary>
            NotsupportMsgNo,
            /// <summary>
            /// データサイズ未設定
            /// </summary>
            NotsetDataSize,
            /// <summary>
            /// データサイズ不一致
            /// </summary>
            UnmatchDataSize,
        }

        /// <summary>
        /// 構造体からバイト配列に変換する
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="structure"></param>
        /// <returns></returns>
        protected static byte[] StructToByteArray<T>(T structure) where T : struct
        {
            int size = Marshal.SizeOf(structure);
            byte[] byteArray = new byte[size];
            GCHandle handle = GCHandle.Alloc(byteArray, GCHandleType.Pinned);

            try
            {
                IntPtr ptr = handle.AddrOfPinnedObject();
                Marshal.StructureToPtr(structure, ptr, false);
            }
            finally
            {
                handle.Free();
            }

            return byteArray;
        }

        /// <summary>
        /// バイト配列から構造体に変換する
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="byeArray"></param>
        /// <returns></returns>
        protected static T ByteArrayToStruct<T>(byte[] byteArray) where T : struct
        {
            T structure = new T();
            GCHandle handle = GCHandle.Alloc(byteArray, GCHandleType.Pinned);

            try
            {
                IntPtr ptr = handle.AddrOfPinnedObject();
                structure = Marshal.PtrToStructure<T>(ptr);
            }
            finally
            {
                handle.Free();
            }

            return structure;
        }

        /// <summary>
        /// メッセージNoの判定
        /// </summary>
        /// <param name="woMsgNo"></param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        protected int JudgMsgNo(UInt16 woMsgNo)
        {
            int inRet = 0;

            // メッセージNoがサポートされているか？
            switch (woMsgNo)
            {
                case (UInt16)MsgNo.ReqReadTag:
                case (UInt16)MsgNo.ResReadTag:
                case (UInt16)MsgNo.ReqWriteTag:
                case (UInt16)MsgNo.ResWriteTag:
                    inRet = (int)ErrCode.Ok;
                    break;

                default:
                    inRet = (int)ErrCode.NotsupportMsgNo;
                    break;
            }

            return inRet;
        }
    }
}
// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が有効になる
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif
