﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;

// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が無効になる(従来通り)
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif
namespace HMI_Container
{
    /// <summary>
    /// メッセージクラス：ACK
    /// </summary>
    public class HMI_MsgAck : HMI_Msg
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_MsgAck()
        {
            InitAck();
        }

        /// <summary>
        /// ACK
        /// </summary>
        [StructLayout(LayoutKind.Sequential, Pack = 1)]
        public struct MsgAck
        {
            /// <summary>
            /// Prefix
            /// </summary>
            public UInt32 m_dwPrefix;

            /// <summary>
            /// メッセージNo
            /// </summary>
            public UInt16 m_woMsgNo;

            /// <summary>
            /// Reserve1
            /// </summary>
            public UInt16 m_woReserve1;

            /// <summary>
            /// エラーコード
            /// </summary>
            public UInt32 m_dwErrCode;

            /// <summary>
            /// Reserve2
            /// </summary>
            public UInt32 m_dwReserve2;
        }

        /// <summary>
        /// ACK情報の初期化
        /// </summary>
        private void InitAck()
        {
            m_clAck.m_dwPrefix = MSG_PREFIX;
            m_clAck.m_woMsgNo= 0;
            m_clAck.m_woReserve1 = 0;
            m_clAck.m_dwStatus = 0;
            m_clAck.m_dwReserve2 = 0;
            //Array.Clear(m_clAck.m_dwReserve2, 0, m_clAck.m_dwReserve2.Length);
            m_byAck = new byte[Marshal.SizeOf<MsgAck>()];
        }

        /// <summary>
        /// ACK情報の更新
        /// </summary>
        /// <param name="woMsgNo"></param>
        /// <param name="dwErrCode"></param>
        public void UpdateAck(UInt16 woMsgNo, UInt32 dwErrCode)
        {
            m_clAck.m_woMsgNo = woMsgNo;
            m_clAck.m_dwStatus = dwErrCode;
        }

        /// <summary>
        /// ACK
        /// </summary>
        public MsgHead m_clAck = new MsgHead();

        /// <summary>
        /// シリアライズされたACK
        /// </summary>
        public byte[] m_byAck;

        /// <summary>
        /// ACKをシリアライズする
        /// </summary>
        public void SerializeAck()
        {
            m_byAck = JsonSerializer.SerializeToUtf8Bytes(m_clAck);
        }

        /// <summary>
        /// ACKをデシリアライズする
        /// </summary>
        public void DeserializeAck()
        {
            m_clAck = JsonSerializer.Deserialize<MsgHead>(m_byAck);
        }

        /// <summary>
        /// ACKの判定
        /// </summary>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public int JudgeAck()
        {
            int inRet = 0;

            // プリフィックスが一致か？
            if (m_clAck.m_dwPrefix != MSG_PREFIX)
            {
                return (int)ErrCode.UnmacthPrefix;
            }

            // メッセージNoがサポートされているか？
            inRet = JudgMsgNo(m_clAck.m_woMsgNo);
            if (inRet != 0)
            {
                return inRet;
            }

            return (int)ErrCode.Ok;
        }

    }
}
// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が有効になる
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif
