﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;
using static HMI_Container.HMI_MngMsgReadTag;

// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が無効になる(従来通り)
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable disable
#else
#endif

namespace HMI_Container
{
    /// <summary>
    /// メッセージヘッダ部クラス
    /// </summary>
    public class HMI_MsgHead : HMI_Msg
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HMI_MsgHead()
        {
            InitHead();
        }

        /// <summary>
        /// ヘッダ部の初期化
        /// </summary>
        public void InitHead()
        {
            m_clHead.m_dwPrefix = MSG_PREFIX;
            m_clHead.m_woMsgNo = 0;
            m_clHead.m_woReserve1 = 0;
            m_clHead.m_dwDataSize = 0;
            m_clHead.m_dwReserve2 = 0;
            //Array.Clear(m_clHead.m_dwReserve2, 0, m_clHead.m_dwReserve2.Length);
            m_byHead = new byte[Marshal.SizeOf(typeof(MsgHead))];
        }

        /// <summary>
        /// ヘッダ部の更新
        /// </summary>
        /// <param name="woMsgNo"></param>
        /// <param name="dwDataSize"></param>
        public void UpdateHead(UInt16 woMsgNo, UInt32 dwDataSize)
        {
            m_clHead.m_woMsgNo = woMsgNo;
            m_clHead.m_dwDataSize = dwDataSize;
        }

        /// <summary>
        /// ヘッダ部
        /// </summary>
        public MsgHead m_clHead = new MsgHead();

        /// <summary>
        /// シリアライズされたヘッダ部
        /// </summary>
        public byte[] m_byHead;

        /// <summary>
        /// ヘッダ部をシリアライズする
        /// </summary>
        public void SerializeHead()
        {
            m_byHead = JsonSerializer.SerializeToUtf8Bytes(m_clHead);
        }

        /// <summary>
        /// ヘッダ部をデシリアライズする
        /// </summary>
        public void DeserializeHead()
        {
            try
            {
                JsonSerializerOptions options = new JsonSerializerOptions
                {
                    DefaultIgnoreCondition = System.Text.Json.Serialization.JsonIgnoreCondition.WhenWritingNull,
                    WriteIndented = true
                };
                m_clHead = JsonSerializer.Deserialize<MsgHead>(m_byHead, options);
            }
            catch (Exception ex)
            {
                string test = ex.Message;
            }
        }

        /// <summary>
        /// ヘッダ部の判定
        /// </summary>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public int JudgeHead()
        {
            int inRet = 0;

            // プリフィックスが一致か？
            if (m_clHead.m_dwPrefix != MSG_PREFIX)
            {
                return (int)ErrCode.UnmacthPrefix;
            }

            // メッセージNoがサポートされているか？
            inRet = JudgMsgNo(m_clHead.m_woMsgNo);
            if (inRet != 0)
            {
                return inRet;
            }

            // データサイズが設定されているか？
            if (m_clHead.m_dwDataSize == 0)
            {
                return (int)ErrCode.NotsetDataSize;
            }

            return (int)ErrCode.Ok;
        }
    }
    /// <summary>
    /// ヘッダ情報
    /// </summary>
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    public class MsgHead
    {
        /// <summary>
        /// Prefix
        /// </summary>
        public UInt32 m_dwPrefix { get; set; }

        /// <summary>
        /// メッセージNo
        /// </summary>
        public UInt16 m_woMsgNo { get; set; }

        /// <summary>
        /// Reserve1
        /// </summary>
        public UInt16 m_woReserve1 { get; set; }

        /// <summary>
        /// データサイズ
        /// </summary>
        public UInt32 m_dwDataSize { get; set; }

        /// <summary>  
        /// Status
        /// </summary>
        public UInt32 m_dwStatus { get; set; }

        /// <summary>
        /// ACK
        /// </summary>
        public UInt16 m_woAck { get; set; }

        /// <summary>
        /// Reserve2
        /// </summary>
        // [MarshalAs(UnmanagedType.ByValArray, SizeConst = 46)]
        // public byte[] m_dwReserve2 = new byte[46];
        // public byte[] m_dwReserve2 { get; set; } = new byte[46];
        public UInt32 m_dwReserve2 { get; set; }
    }
}
// .net core 3.0以上 または .NET 5以上 ここ以下の行はnull許容参照型が有効になる
#if NETCOREAPP || NET5_0_OR_GREATER
#nullable restore
#else
#endif
