﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using EdgeCode; // RT-edge template code

namespace HMI_Serv
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        edgeCode EgProc = null;                 //RTedge フレームワーク+APIを使用したコードクラス

        public MainWindow(string[] args)
        {
            bool debugF = false;
            //引数に -Debugがある場合はデバッグとして動作する
            if (args != null && args.Contains("-Debug"))
            {
                debugF = true;
            }
#if DEBUG
debugF = true;
#endif
            InitializeComponent();              //Componentの初期化(自動生成コード)
            if (debugF == true)
                AllocConsole();                     //[DEBUG用] コンソール出力用
            EgProc = new edgeCode();            //RTedge処理クラスの作成
            EgProc.mInstMainClose = Stop;       //フレームワーク内終了メッセージ処理時の関数指定
            EgProc.InitService();               //RTedge処理クラスの初期化
        }

        /// <summary>
        /// サービス終了処理
        /// </summary>
        public void Stop()
        {
            try
            {
                Dispatcher.Invoke((Action)(() =>
                {
                    Close();                        //Window Close関数呼び出し
                }));
            }
            catch (Exception ex)
            {
                EgProc.LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            ;
        }

        /// <summary>
        /// Windowクローズ処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            try
            {
                EgProc.KillService();           //サービス削除関数呼び出し<必須>
                EgProc.EGAPI.EgFinalize();      //APIのFinalize処理追加 <必須>
                EgProc.Release();               //後処理
                //[TODO] : 必要に応じて終了処理追加
            }
            catch (Exception ex)
            {
                EgProc.LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        /// <summary>
        /// [DEBUG用] コンソール出力するための定義
        /// </summary>
        /// <returns></returns>
        [DllImport("kernel32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool AllocConsole();
    }
}
