﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Threading.Tasks;
using RTedge;   // RT-edge system API
using System.Runtime.CompilerServices;
using System.Threading;
using HMI_Container;
using HMI_Serv;
using System.IO.Pipes;
using System.Runtime.InteropServices;
using System.IO;
using System.Security.AccessControl;                // HMIコンテナクラス


namespace EdgeCode
{
    /// <summary>
    /// RT-edge API/フレームワーク 各種値定義クラス
    /// </summary>
    public class EDGECONST
    {
        /// <summary>
        /// 各種定義追加
        /// </summary>

        public const string TAG_PREFIX = "SERVICE.";            // サービスプロパティタグ名先頭に付与する文字列
        public const bool SERVICE_UP = true;                    // 起動中
        public const bool SERVICE_DOWN = false;                 // 停止中
        public const int MODE_AUTO = 1;                         // 動作モード 入出自動サイクル
        public const int MODE_SEMIAUTO = 2;                     // 動作モード 入力サイクル＋出力デマンド
        public const int MODE_MANUAL = 3;                       // 動作モード 入出力デマンド
        public const int DEFAULT_MODE = MODE_SEMIAUTO;          // 動作モード デフォルト値
        public const int DEFAULT_CYCLE = 1000;                  // 動作サイクル デフォルト値
        public const int DEFAULT_INPRIO = 148;                  // Edgeシステムへの入力サービススレッドプライオリティ デフォルト値
        public const int DEFAULT_OUTPRIO = 149;                 // Edgeシステムからの出力サービススレッドプライオリティ デフォルト値
        ////
        /// サービス名
        //
        public string SERVICENAME;                              // サービス名 ※ECIファイル(XML)から取得
        ////
        /// サービスインジケータ名
        //        
        public string IND_MYSERV_STATUS;                        // サービス起動
        public string IND_MYSERV_ERROR;                         // サービスエラー発生中
        public string IND_MYSERV_RUN;		                    // サービス実行中
        public string IND_MYSERV_LIVE;                          // サービス実行カウンタ
        ////
        /// サービスプロパティタグ名
        //        
        public string PROP_MYSERV_MODE;                         // サービス動作モード
        public string PROP_MYSERV_CYCLE;                        // サービス動作サイクル(ms)
        public string PROP_MYSERV_INPRIO;                       // Edgeシステムへの入力サービススレッドプライオリティ
        public string PROP_MYSERV_OUTPRIO;                      // Edgeシステムからの出力サービススレッドプライオリティ	
        ////
        /// サービス提供タグ名
        //  [TODO] : 任意のタグ追加 (XMLで定義することを推奨)
        // public static string TAG_MYSERV_XXX = "XXX";
        ////
        /// サービス固有メッセージ
        //  [TODO] : 任意のメッセージID追加(ユーザメッセージ 20000～29999番指定可)
        // public static int EM_SERVICE_XXX = 20000;    //任意の番号
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="Name"></param>
        public EDGECONST()
        { }

        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="Name"></param>
        public void SET_EDGECONST(string Name)
        {
            // NOTE: サービス名から各タグ名称を生成します。
            SERVICENAME = Name;
            IND_MYSERV_STATUS = TAG_PREFIX + SERVICENAME + ".Status";           // サービス起動
            IND_MYSERV_ERROR = TAG_PREFIX + SERVICENAME + ".Error";             // サービスエラー発生中
            IND_MYSERV_RUN = TAG_PREFIX + SERVICENAME + ".Run";                 // サービス実行中
            IND_MYSERV_LIVE = TAG_PREFIX + SERVICENAME + ".Live";               // サービス実行カウンタ

            PROP_MYSERV_MODE = TAG_PREFIX + SERVICENAME + ".Mode";              // サービス動作モード
            PROP_MYSERV_CYCLE = TAG_PREFIX + SERVICENAME + ".Cycle";            // Adviseタグ監視周期(ms)
            PROP_MYSERV_INPRIO = TAG_PREFIX + SERVICENAME + ".InPriority";      // Edgeシステムへの入力サービススレッドプライオリティ
            PROP_MYSERV_OUTPRIO = TAG_PREFIX + SERVICENAME + ".OutPriority";    // Edgeシステムからの出力サービススレッドプライオリティ	
        }
    }
    /// <summary>
    /// RT-edge API/フレームワーク実装クラス
    /// </summary>
    public class edgeCode
    {
        ////
        /// サービス内部状態
        //
        public class MYSTATE
        {
            public bool State;                  // インジケータ.エッジシステム接続状態
            public bool Run;                    // インジケータ.サービス実行状態
            public bool Error;                  // インジケータ.サービス異常状態
            public UInt32 Live;                 // インジケータ.サービス実行カウンタ
            public UInt64 StartTime;            // インジケータ.サービス開始時のタイムスタンプ
            public byte Mode;                   // プロパティ.サービス実行モード
            public UInt32 Cycle;                // プロパティ.サービス実行サイクル(ms)	[TODO] : 必要に応じて実装
            public byte InPriority;             // プロパティ.サービス(入力)プライオリティ	[TODO] : 必要に応じて実装
            public byte OutPriority;            // プロパティ.サービス(出力)プライオリティ	[TODO] : 必要に応じて実装
            public SemaphoreSlim hInTrigSem;    // 入力サービストリガセマフォハンドル	[TODO] : 必要に応じて実装
            public SemaphoreSlim hOutTrigSem;   // 出力サービストリガセマフォハンドル	[TODO] : 必要に応じて実装
            //[TODO] : 任意のステータス追加
            public MYSTATE()
            {
                State = false;
                Run = false;
                Error = false;
                Live = 0;
                StartTime = 0;
                Mode = 0;
                Cycle = 1000;
                InPriority = 149;
                OutPriority = 148;
                hInTrigSem = null;
                hOutTrigSem = null;
            }
        }

        ////
        /// edge_APIクラス インスタンス化 
        //
        public edge_API EGAPI = null;                               //edge APIのインスタンス
        public MYSTATE STATE = null;                                //サービス状態クラスのインスタンス
        public EDGECONST EGCONST = null;                            // 各種定義値
        ////
        /// スレッド変数 
        //
        private CancellationTokenSource mThreadAutoTS = null;       //自動更新用スレッド       
        private CancellationTokenSource mThreadInServiceTS = null;  //入力スレッドキャンセル処理用変数
        private CancellationTokenSource mThreadOutServiceTS = null; //出力スレッドキャンセル処理用変数
        ////
        /// メイン処理終了用コールバック関数
        //
        public delegate void CallbackMainClose();
        public CallbackMainClose mInstMainClose = null;
        //
        // 名前付きパイプ通信情報
        //
        public NamedPipeServerStream m_clPipeIn = null;             // 名前付きパイプ通信クラス：入力処理
        public NamedPipeServerStream m_clPipeOut = null;            // 名前付きパイプ通信クラス：出力処理
        public HMI_MngMsg m_clMngMsgIn = null;                      // メッセージ通信クラス：入力処理
        public HMI_MngMsg m_clMngMsgOut = null;                     // メッセージ通信クラス：出力処理
        public HMI_MngMsgReadTag m_clMngMsgReadTag = null;          // タグ読み込み管理クラス
        public HMI_MngMsgWriteTag m_clMngMsgWriteTag = null;        // タグ書き込み管理クラス
        /// <summary>
        /// 名前付きパイプ通信処理ステージ
        /// </summary>
        public enum StagePipe
        {
            /// <summary>
            /// なし
            /// </summary>
            None =0,
            /// <summary>
            /// 名前付きパイプクラスの生成
            /// </summary>
            Create,
            /// <summary>
            /// 接続
            /// </summary>
            Connect,
            /// <summary>
            /// 要求伝文(ヘッダ部)受信
            /// </summary>
            RecvReqHead,
            /// <summary>
            /// 要求伝文(ヘッダ部)判定
            /// </summary>
            JudgeReqHead,
            /// <summary>
            /// 要求伝文(ヘッダ部)ACK
            /// </summary>
            SendReqHeadAck,
            /// <summary>
            /// 要求伝文(データ部)受信：タグ読み込み
            /// </summary>
            RecvReqDataReadTag,
            /// <summary>
            /// 要求伝文(データ部)受信：タグ書き込み
            /// </summary>
            RecvReqDataWriteTag,
            /// <summary>
            /// 要求伝文(データ部)ACK
            /// </summary>
            SendReqDataAck,
            /// <summary>
            /// 要求伝文(データ部)ACK判定
            /// </summary>
            JudgeReqDataAck,
            /// <summary>
            /// 応答伝文作成：タグ読み込み
            /// </summary>
            MakeResReadTag,
            /// <summary>
            /// 応答伝文作成：タグ書き込み
            /// </summary>
            MakeResWriteTag,
            /// <summary>
            /// 応答伝文(ヘッダ部)送信
            /// </summary>
            SendResHead,
            /// <summary>
            /// 応答伝文(ヘッダ部)ACK受信
            /// </summary>
            RecvResHeadAck,
            /// <summary>
            /// 応答伝文(ヘッダ部)ACK判定
            /// </summary>
            JudgeResHeadAck,
            /// <summary>
            /// 応答伝文(データ部)送信
            /// </summary>
            SendResData,
            /// <summary>
            /// 応答伝文(データ部)ACK受信
            /// </summary>
            RecvResDataAck,
            /// <summary>
            /// 応答伝文(データ部)ACK判定
            /// </summary>
            JudgeResDataAck,
            /// <summary>
            /// 切断
            /// </summary>
            Disconnect,
        }
        /// <summary>
        /// 最大処理ステージ回数
        /// </summary>
        public const int m_inStagePipeCntMax = 100;
        /// <summary>
        /// 通信読み込み処理ﾀｲﾑｱｳﾄ
        /// </summary>
        public const int timeoutMilliseconds = 600000;  // タイムアウト時間（ミリ秒）1分を設定 (仮)

        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="myMsgHndlers">メッセージハンドラー関数を指定</param>
        public edgeCode() 
        {
            EGAPI = new edge_API();
            STATE = new MYSTATE();
            EGCONST = new EDGECONST();
            m_clMngMsgIn = new HMI_MngMsg();
            m_clMngMsgOut = new HMI_MngMsg();
            m_clMngMsgReadTag = new HMI_MngMsgReadTag();
            m_clMngMsgWriteTag = new HMI_MngMsgWriteTag();
    }
        /// <summary>
        /// 後処理処理(デストラクタ)
        /// [TODO] : 必要に応じて変更
        /// </summary>
        public void Release()
        {
            try
            {
                ClosePipe(ref m_clPipeIn);          // HMIサーバー通信クラス：入力処理の破棄
                ClosePipe(ref m_clPipeOut);         // HMIサーバー通信クラス：出力処理の破棄

                if (mThreadInServiceTS != null)     //入力スレッド終了処理
                {
                    mThreadInServiceTS.Cancel();    //キャンセル状態をセットし、スレッド処理で抜ける
                    mThreadInServiceTS = null;
                }

                if (mThreadOutServiceTS != null)    //出力スレッド終了処理
                {
                    mThreadOutServiceTS.Cancel();   //キャンセル状態をセットし、スレッド処理で抜ける
                    mThreadOutServiceTS = null;
                }

                if (mThreadAutoTS != null)          //自動更新スレッド終了処理
                {
                    mThreadAutoTS.Cancel();         //キャンセル状態をセットし、スレッド処理で抜ける
                    mThreadAutoTS = null;
                }
                
                if (STATE.hInTrigSem != null)
                    STATE.hInTrigSem.Dispose();
                if (STATE.hOutTrigSem != null)
                    STATE.hOutTrigSem.Dispose();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        
        /// <summary>
        /// Edgeメッセージハンドラ
        /// [TODO] : 必要に応じてメッセージ・処理を追加
        /// </summary>
        /// <param name="senderName">メッセージ送信元</param>
        /// <param name="messNo">メッセージ番号</param>
        /// <param name="paramArray">メッセージパラメータ</param>
        /// <returns></returns>
        object WMLOCK = new object();
        public Int32 MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                int result = 0;
                lock (WMLOCK) //排他制御 頻繁にやり取りする場合用 不要の場合は削除
                {
                    switch (messNo) //メッセージ番号毎に処理
                    {
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:    //サービス開始指令
                            result = StartService();
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:   // サービス終了指令
                            {
                                if (mInstMainClose != null)
                                    mInstMainClose();               // コールバック関数からは戻りません
                                else
                                    result = KillService();
                            }
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:  // サービス一時停止指令
                            result = PauseService();
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE: // サービスアップデート指令
                            switch ((int)STATE.Mode)
                            {
                                case EDGECONST.MODE_AUTO:
                                    break;
                                case EDGECONST.MODE_SEMIAUTO:
                                    KickOutput();
                                    break;
                                case EDGECONST.MODE_MANUAL:
                                    KickInput();
                                    KickOutput();
                                    break;
                                default:
                                    break;
                            }
                            break;
                        //
                        // [TODO] : 任意のメッセージ処理を追加
                        //
                        default:
                            break;
                    }
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// Edgeシステム入出力自動サイクルスレッド
        /// 必要が無ければ変更不要
        /// </summary>
        /// <remarks>
        /// このコンテナでは、
        /// クライアントから接続または送信されるまで待機するため、
        /// 入力/出力スレッドに対して、トリガ通信しない。
        /// 入力/出力スレッドは実行状態を判断して処理を行う
        /// </remarks>
        public async void ThreadAutoCycle(CancellationToken cancelToken)
        {
            try
            {
				await Task.Run(() =>
                {
                    try
                    {

                        while (true)
                        {
                            if (cancelToken.IsCancellationRequested)        // スレッドを抜ける処理
                                break;

                            ++STATE.Live;                                   // サービス実行カウンタ増加
                            GetProperty();                                  // サービスプロパティのリロード

                            if (STATE.Run)                                  // サービス実行状態なら処理する
                            {
                                switch (STATE.Mode)                         // サービス動作モードに応じて分岐
                                {
                                    case EDGECONST.MODE_AUTO:               // 自動サイクルモード時（入出力共に自動更新する）
                                        break;
                                    case EDGECONST.MODE_SEMIAUTO:           // 半自動サイクル時（入力のみ自動更新、出力はEM_SERVICE_UPDATEで指令）
                                        break;
                                    case EDGECONST.MODE_MANUAL:             // 手動サイクル時（入出力はEM_SERVICE_UPDATEで指令するので無処理）
                                        break;
                                }
                            }

                            UpdateIndicator();                              // サービスインジケータ更新
                            System.Threading.Thread.Sleep((int)STATE.Cycle);// サイクル待ち
                        }
                    }
                    catch (Exception ex)
                    {
                        LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                    }
                });
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// データをサイズ関係なく読み取る処理を独自に作成 
        /// </summary>
        /// <param name="pipeServer"></param>
        /// <param name="retBuffer"></param>
        /// <param name="cancellationToken"></param>
        /// <returns></returns>
        public static async Task<byte[]> ReadPipeWithCancel(NamedPipeServerStream pipeServer)
        {
            using (CancellationTokenSource ctsUseTimer = new CancellationTokenSource())
            {
                // タイムアウト時間後にキャンセルするように設定
                ctsUseTimer.CancelAfter(timeoutMilliseconds);

                using (MemoryStream ms = new MemoryStream())
                {
                    byte[] buffer = new byte[1024];  // バッファは1024バイトに設定
                    int bytesRead;
                    try
                    {
                        // データが送られてくるまで読み取る
                        while (pipeServer.IsConnected)  // クライアントが接続されている限りループ
                        {
                            // 非同期でデータを読み取り、キャンセルトークンを指定
                            bytesRead = await pipeServer.ReadAsync(buffer, 0, buffer.Length, ctsUseTimer.Token);

                            if (bytesRead > 0)
                            {
                                // データがある場合、メモリストリームに書き込む
                                ms.Write(buffer, 0, bytesRead);
                            }

                            if (bytesRead < buffer.Length)
                            {
                                // バッファサイズよりも少ないデータが読み取られたら終了
                                if (bytesRead == 0)
                                {
                                    Console.WriteLine("ReadPipeWithCancel bytesRead Size == 0");
                                }
                                break;
                            }
                        }

                        // すべてのデータを受信し終わったら、バイト配列に変換
                        return ms.ToArray();

                    }
                    catch (IOException ex)
                    {
                        // I/O エラーが発生した場合の処理
                        Console.WriteLine($"I/O error: {ex.Message}");
                        return new byte[0];
                    }
                    catch (OperationCanceledException)
                    {
                        Console.WriteLine("Operation was canceled.");
                        return new byte[0];
                    }

                }
            }
        }
        /// <summary>
        /// Edgeシステムへのデータ入力
        /// [TODO] : 必要に応じて値を書き込むタグと値を指定
        /// </summary>
        /// <remarks>
        /// このコンテナでは、
        /// クライアントから接続または送信されるまで待機するため、
        /// 入力/出力スレッドに対して、トリガ通信しない。
        /// 入力/出力スレッドは実行状態を判断して処理を行う
        /// </remarks>
        public async void ThreadInService(CancellationToken cancelToken)
        {
            // ログ出力時に付加するサービス名(タスク名)
            string srLogPrefix = "[InService]";
            // ★TODO 時間を決める・別定義にする必要あり
            bool ConnectSuccessFlag = true;
            try
            {
                int result = 0;
                while (true)
                {
                    // キャンセルトークンが来たら終了
                    if (cancelToken.IsCancellationRequested)
                        break;

                    // 実行状態が停止の場合は、ループの先頭へ
                    if (false == STATE.Run)
                    {
                        Thread.Sleep((int)STATE.Cycle);
                        continue;
                    }

                    //タスク処理を開始する
                    try
                    {
                        // 処理ステージの初期化
                        int enStagePipe = (int)StagePipe.Create;
                        // 戻り値
                        UInt32 dwRet = 0;
                        // 要求/応答伝文判定結果
                        int inMsgResult = 0;
                        // 読み込みサイズ
                        int inReadSize = 0;

                        int edgeResult = 0;
                        // ============================================================
                        // 処理ステージを移動しながら、処理を行う
                        // ============================================================
                        for (int inStageCnt = 0; inStageCnt < m_inStagePipeCntMax; inStageCnt++)
                        {
                            // ============================================================
                            // 該当処理の実行
                            // ============================================================
                            switch (enStagePipe)
                            {
                                // ============================================================
                                // 名前付きパイプクラスの生成
                                // ============================================================
                                case (int)StagePipe.Create:
                                    try
                                    {
                                        ConnectSuccessFlag = true;

                                        // 名前付きパイプクラスの生成判定
                                        if (null == m_clPipeIn)
                                        {
                                            LOG(srLogPrefix + "Create pipe ...");
                                            // 名前付きパイプクラスの生成
                                            dwRet = CreatePipe(HMI_Static.HMI_PIPE_IN_NAME, ref m_clPipeIn);
                                        }
                                        else
                                        {
                                            // 生成済みは正常終了と判断
                                            dwRet = 0;
                                        }
                                        // 生成結果判定
                                        if (0 == dwRet)
                                        {
                                            // 生成成功
                                            enStagePipe = (int)StagePipe.Connect;          // ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 生成失敗
                                            LOG(srLogPrefix + "Failed: create pipe[" + dwRet.ToString("X8") + "]");
                                            inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                        }
                                    }
                                    catch (IOException ex)
                                    {
                                        LOG(srLogPrefix + "Failed: StagePipe.Create Fatal. ex = " + ex.Message);
                                        inStageCnt = m_inStagePipeCntMax;
                                    }
                                    break;

                                // ============================================================
                                // 接続
                                // ============================================================
                                case (int)StagePipe.Connect:
                                    // 接続状態判定
                                    ConnectSuccessFlag = true;
                                    try
                                    {
                                        //そもそもnullの場合は失敗と断定
                                        if (m_clPipeIn == null)
                                        {
                                            ConnectSuccessFlag = false;
                                        }
                                        else
                                        {
                                            //接続していない場合は接続処理
                                            if (!m_clPipeIn.IsConnected)
                                            {
                                                LOG(srLogPrefix + "Wait for client connection ...");
                                                // 接続待ち
                                                try
                                                {
                                                    await m_clPipeIn.WaitForConnectionAsync(cancelToken);
                                                    LOG(srLogPrefix + "Client connected.");
                                                }
                                                catch (IOException ex)
                                                {
                                                    Console.WriteLine("StagePipe.Connect パイプが壊れました。再接続を試みます: " + ex.Message);
                                                    ConnectSuccessFlag = false;
                                                    // 再接続処理など
                                                }

                                            }
                                        }
                                    }
                                    catch (IOException ex)
                                    {
                                        ConnectSuccessFlag = false;
                                        LOG(srLogPrefix + "Failed: StagePipe.Connect Fatal. ex = " + ex.Message);
                                    }
                                    if (ConnectSuccessFlag == true)
                                        enStagePipe = (int)StagePipe.RecvReqHead;           // ▼次のステージへ
                                    else
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    break;

                                // ============================================================
                                // 要求伝文(ヘッダ部)受信
                                // ============================================================
                                case (int)StagePipe.RecvReqHead:
                                    try
                                    {
                                        // 要求伝文(ヘッダ部)をシリアライズする
                                        m_clMngMsgIn.m_clReqHead.SerializeHead();
                                        // 要求伝文(ヘッダ部)の読み込み
                                        m_clMngMsgIn.m_clReqHead.m_byHead = await ReadPipeWithCancel(m_clPipeIn);
                                        if (m_clMngMsgIn.m_clReqHead.m_byHead.Length > 0)
                                        {
                                            // 読み込み成功
                                            enStagePipe = (int)StagePipe.JudgeReqHead;      // ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 読み込み失敗
                                            LOG(srLogPrefix + "Unmatch req head size [" + m_clMngMsgIn.m_clReqHead.m_byHead.Length.ToString() + "][" + inReadSize.ToString() + "] _ RecvReqHead IN");
                                            enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                        }
                                    }
                                    catch (IOException ex)
                                    {
                                        Console.WriteLine("StagePipe.RecvReqHead クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("StagePipe.RecvReqHead エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    break;

                                // ============================================================
                                // 要求伝文(ヘッダ部)判定
                                // ============================================================
                                case (int)StagePipe.JudgeReqHead:
                                    try
                                    {
                                        // 要求伝文(ヘッダ部)をデシリアライズする
                                        m_clMngMsgIn.m_clReqHead.DeserializeHead();
                                        // 要求伝文(ヘッダ部)を判定する
                                        inMsgResult = m_clMngMsgIn.m_clReqHead.JudgeHead();
                                        if (inMsgResult != 0)
                                        {
                                            // 要求伝文(ヘッダ部)判定結果NG
                                            LOG(srLogPrefix + "Unmacth ReqHead [" + inMsgResult.ToString() + "]");
                                        }
                                        enStagePipe = (int)StagePipe.SendReqHeadAck;      // ▼次のステージへ
                                    }
                                    catch (IOException ex)
                                    {
                                        Console.WriteLine("StagePipe.JudgeReqHead クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("StagePipe.JudgeReqHead エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    break;

                                // ============================================================
                                // 要求伝文(ヘッダ情報)ACK送信
                                // ============================================================
                                case (int)StagePipe.SendReqHeadAck:
                                    try
                                    {
                                        // 要求伝文(ヘッダ情報)ACK作成
                                        m_clMngMsgIn.m_clReqHeadAck.UpdateAck(m_clMngMsgIn.m_clReqHead.m_clHead.m_woMsgNo, (UInt32)inMsgResult);
                                        // 要求伝文(ヘッダ情報)ACKシリアライズ
                                        m_clMngMsgIn.m_clReqHeadAck.SerializeAck();
                                        // 要求伝文(ヘッダ情報)ACK送信
                                        await m_clPipeIn.WriteAsync(m_clMngMsgIn.m_clReqHeadAck.m_byAck, 0, m_clMngMsgIn.m_clReqHeadAck.m_byAck.Length, cancelToken);
                                        // 要求伝文(ヘッダ部)判定結果によって次のステージを決定
                                        if (inMsgResult == 0)
                                        {
                                            // 要求伝文(ヘッダ部)判定結果OK
                                            // メッセージNoによる分岐
                                            switch (m_clMngMsgIn.m_clReqHead.m_clHead.m_woMsgNo)
                                            {
                                                // タグデータ読み込み要求
                                                case (UInt16)HMI_Msg.MsgNo.ReqReadTag:
                                                    enStagePipe = (int)StagePipe.RecvReqDataReadTag;        // ▼次のステージへ
                                                    break;

                                                // 上記以外はエラー
                                                default:
                                                    LOG(srLogPrefix + "Unknown MsgNo [" + m_clMngMsgIn.m_clReqHead.m_clHead.m_woMsgNo.ToString() + "]");
                                                    inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                                    break;
                                            }

                                        }
                                        else
                                        {
                                            // 要求伝文(ヘッダ部)判定結果NG
                                            inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                        }
                                    }
                                    catch (IOException ex)
                                    {
                                        Console.WriteLine("StagePipe.SendReqHeadAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("StagePipe.SendReqHeadAck エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    break;

                                // ============================================================
                                // 要求伝文(データ部)受信：タグ読み込み
                                // ============================================================
                                case (int)StagePipe.RecvReqDataReadTag:
                                    try
                                    {
                                        // 要求伝文(データ部)の確保
                                        m_clMngMsgReadTag.AllocReqData(m_clMngMsgIn.m_clReqHead.m_clHead.m_dwDataSize);
                                        // 要求伝文(データ部)の読み込み
                                        m_clMngMsgReadTag.m_byReqData = await ReadPipeWithCancel(m_clPipeIn);
                                        if (m_clMngMsgReadTag.m_byReqData.Length > 0)
                                        {
                                            // 読み込み成功
                                            inMsgResult = 0;
                                            // 要求伝文(データ部)をデシリアライズする
                                            m_clMngMsgReadTag.DeserializeReqData();
                                            enStagePipe = (int)StagePipe.SendReqDataAck;      // ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 読み込み失敗
                                            inMsgResult = (int)HMI_Msg.ErrCode.UnmatchDataSize;
                                            LOG(srLogPrefix + "Unmatch req data size [" + m_clMngMsgReadTag.m_byReqData.Length.ToString() + "][" + inReadSize.ToString() + "] _ RecvReqDataReadTag");
                                            enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                        }
                                        
                                    }
                                    catch (IOException ex)
                                    {
                                        Console.WriteLine("StagePipe.RecvReqDataReadTag クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 要求伝文(データ部)ACK送信
                                // ============================================================
                                case (int)StagePipe.SendReqDataAck:
                                    try
                                    {
                                        // 要求伝文(データ情報)ACK作成
                                        m_clMngMsgIn.m_clReqDataAck.UpdateAck(m_clMngMsgIn.m_clReqHead.m_clHead.m_woMsgNo, (UInt32)inMsgResult);
                                        // 要求伝文(データ情報)ACKシリアライズ
                                        m_clMngMsgIn.m_clReqDataAck.SerializeAck();
                                        // 要求伝文(データ情報)ACK送信
                                        await m_clPipeIn.WriteAsync(m_clMngMsgIn.m_clReqDataAck.m_byAck, 0, m_clMngMsgIn.m_clReqDataAck.m_byAck.Length, cancelToken);
                                        // 要求伝文(ヘッダ部)判定結果によって次のステージを決定
                                        if (inMsgResult == 0)
                                        {
                                            // 要求伝文(ヘッダ部)判定結果OK
                                            // メッセージNoによる分岐
                                            switch (m_clMngMsgIn.m_clReqHead.m_clHead.m_woMsgNo)
                                            {
                                                // タグデータ読み込み要求
                                                case (UInt16)HMI_Msg.MsgNo.ReqReadTag:
                                                    enStagePipe = (int)StagePipe.MakeResReadTag;        // ▼次のステージへ
                                                    break;

                                                // 上記以外はエラー
                                                default:
                                                    LOG(srLogPrefix + "Unknown MsgNo [" + m_clMngMsgIn.m_clReqHead.m_clHead.m_woMsgNo.ToString() + "]");
                                                    inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                                    break;
                                            }

                                        }
                                        else
                                        {
                                            // 要求伝文(ヘッダ部)判定結果NG
                                            inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                        }
                                    }
                                    catch (IOException ex)
                                    {
                                        Console.WriteLine("StagePipe.SendReqDataAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 応答伝文(データ部)作成：タグ読み込み
                                // ============================================================
                                case (int)StagePipe.MakeResReadTag:
                                    try
                                    {
                                        // 応答伝文(データ部)の確保
                                        m_clMngMsgReadTag.AllocReqData(m_clMngMsgIn.m_clReqHead.m_clHead.m_dwDataSize);
                                        // Itemを空にする
                                        m_clMngMsgReadTag.m_clResData.m_lsTags.Clear();
                                        // RT-edgeの読み込み
                                        object value = new object();
                                        object objType = new object();
                                        object objSize = new object();
                                        foreach (HMI_MngMsgReadTag.MsgDataTagName tagvalue in m_clMngMsgReadTag.m_clReqData.m_lsTags)
                                        {

                                            edgeResult = EGAPI.EgTagGetProperty(tagvalue.m_srName, "Type", ref objType);   //タグのTypeを取得しMsgDataTagValueに格納
                                            if (edgeResult == 0)
                                            {
                                                // stringの場合は別処理
                                                if ((ushort)objType == (ushort)egTagDataType.String)
                                                {
                                                    edgeResult = EGAPI.EgTagGetProperty(tagvalue.m_srName, "Size", ref objSize);
                                                    if (edgeResult == 0 && null != objSize && 0 != (ushort)objSize)//Tagのサイズを取得できた場合はstringの読み込みを行う
                                                    {
                                                        string strValue = "";
                                                        edgeResult = EGAPI.EgTagReadString(tagvalue.m_srName, ref strValue, (ushort)objSize);
                                                        value = strValue;
                                                    }
                                                }
                                                else
                                                {
                                                    edgeResult = EGAPI.EgTagRead(tagvalue.m_srName, ref value);
                                                }
                                                if (edgeResult == 0) //上記APIが成功したときのみ格納
                                                {
                                                    var clItem = new MsgDataTagValue();
                                                    clItem.m_srName = tagvalue.m_srName;
                                                    clItem.m_woType = (ushort)objType;
                                                    clItem.m_obVal = MsgDataTagValue.ConvertValue(clItem.m_woType, value);
                                                    m_clMngMsgReadTag.m_clResData.m_lsTags.Add(clItem);
                                                }
                                            }
                                            if (edgeResult != 0)
                                                result |= edgeResult;
                                        }
                                        // 応答伝文(データ部)をシリアライズする
                                        m_clMngMsgReadTag.SerializeResData();
                                        // 応答伝文(データ情報)を判定する
                                        if (m_clMngMsgReadTag.m_byResData.Length == 0)
                                        {
                                            // 応答伝文(データ情報)作成失敗
                                            inMsgResult = (int)HMI_Msg.ErrCode.UnmatchDataSize;
                                            // ログにエラーメッセージを表示
                                            LOG(srLogPrefix + "Failed: create data[" + inMsgResult.ToString("X8") + "]");
                                        }
                                        enStagePipe = (int)StagePipe.SendResHead; // ▼次のステージへ
                                    }
                                    catch (IOException ex) //「応答伝文(データ部)作成：タグ読み込み」で例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.MakeResReadTag クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 応答伝文(ヘッダ部)送信
                                // ============================================================
                                case (int)StagePipe.SendResHead:
                                    try
                                    {
                                        // 応答伝文(ヘッダ部)作成
                                        m_clMngMsgIn.m_clResHead.UpdateHead((UInt16)HMI_Msg.MsgNo.ResReadTag, (UInt32)m_clMngMsgReadTag.m_byResData.Length);
                                        // 応答伝文(ヘッダ部)シリアライズ
                                        m_clMngMsgIn.m_clResHead.SerializeHead();
                                        // 応答伝文(ヘッダ部)送信
                                        await m_clPipeIn.WriteAsync(m_clMngMsgIn.m_clResHead.m_byHead, 0, m_clMngMsgIn.m_clResHead.m_byHead.Length, cancelToken);

                                        if (inMsgResult == 0)
                                        {
                                            // 応答伝文(ヘッダ部)判定結果OK

                                            if (m_clMngMsgIn.m_clResHead.m_byHead.Length != 0)
                                            {
                                                // 応答伝文(ヘッダ部)送信成功
                                                enStagePipe = (int)StagePipe.RecvResHeadAck; // ▼次のステージへ
                                            }
                                            else
                                            {
                                                // ログにエラーメッセージを表示
                                                // 応答伝文(ヘッダ部)送信失敗
                                                inStageCnt = m_inStagePipeCntMax;            // ▼処理ステージ終了
                                            }
                                        }
                                        else
                                        {
                                            // 応答伝文(ヘッダ部)判定結果NG
                                            inStageCnt = m_inStagePipeCntMax;                // ▼処理ステージ終了
                                        }
                                    }
                                    catch (IOException ex)//「応答伝文(ヘッダ部)送信」で例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.SendResHeadクライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 応答伝文(ヘッダ部)ACK受信
                                // ============================================================
                                case (int)StagePipe.RecvResHeadAck:
                                    try
                                    {
                                        // 応答伝文(ヘッダ部)ACKをシリアライズする
                                        m_clMngMsgIn.m_clResHeadAck.SerializeAck();
                                        // 応答伝文(ヘッダ部)ACKの読み込み
                                        m_clMngMsgIn.m_clResHeadAck.m_byAck = await ReadPipeWithCancel(m_clPipeIn);
                                        if (m_clMngMsgIn.m_clResHeadAck.m_byAck.Length > 0)
                                        {
                                            // 応答伝文(ヘッダ部)ACK読み込み成功
                                            enStagePipe = (int)StagePipe.JudgeResHeadAck;　// ▼次のステージへ　
                                        }
                                        else
                                        {
                                            // 応答伝文(ヘッダ部)ACK読み込み失敗
                                            LOG(srLogPrefix + "Unmatch res head ack size [" + m_clMngMsgIn.m_clResHeadAck.m_byAck.ToString() + "][" + inReadSize.ToString() + "] _ RecvResHeadAck");
                                            enStagePipe = (int)StagePipe.Disconnect;　   // ▼次のステージへ　
                                        }
                                    }
                                    catch (IOException ex)//「応答伝文(ヘッダ部)ACK受信」で例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.RecvResHeadAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 応答伝文(ヘッダ部)ACK判定
                                // ============================================================
                                case (int)StagePipe.JudgeResHeadAck:
                                    try
                                    {
                                        // 応答伝文(ヘッダ部)ACKをデシリアライズする
                                        m_clMngMsgIn.m_clResHeadAck.DeserializeAck();
                                        // 応答伝文(ヘッダ部)ACKを判定する
                                        inMsgResult = m_clMngMsgIn.m_clResHeadAck.JudgeAck();
                                        if (inMsgResult == 0)
                                        {
                                            // 応答伝文(ヘッダ部)ACK判定結果OK
                                            enStagePipe = (int)StagePipe.SendResData;      // ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 応答伝文(ヘッダ部)ACK判定結果NG
                                            LOG(srLogPrefix + "Unmacth ReqHead [" + inMsgResult.ToString() + "] _ JudgeResHeadAck");
                                            enStagePipe = (int)StagePipe.Disconnect;
                                            inStageCnt = m_inStagePipeCntMax;           // ▼処理ステージ終了
                                        }
                                    }
                                    catch (IOException ex)    //「応答伝文(ヘッダ部)ACK受信」で例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.JudgeResHeadAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 応答伝文(データ部)送信
                                // ============================================================
                                case (int)StagePipe.SendResData:
                                    try
                                    {
                                        // 応答伝文(データ情報)送信
                                        await m_clPipeIn.WriteAsync(m_clMngMsgReadTag.m_byResData, 0, m_clMngMsgReadTag.m_byResData.Length, cancelToken);

                                        if (m_clMngMsgReadTag.m_byResData.Length > 0)
                                        {
                                            // 応答伝文(データ情報)送信成功
                                            enStagePipe = (int)StagePipe.RecvResDataAck;　// ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 応答伝文(データ情報)送信失敗
                                            enStagePipe = (int)StagePipe.Disconnect;　   // ▼次のステージへ　　
                                        }
                                    }
                                    catch (IOException ex)//「応答伝文(データ部)送信」で例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.SendResData クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 応答伝文(データ部)ACK受信
                                // ============================================================
                                case (int)StagePipe.RecvResDataAck:
                                    try
                                    {
                                        // 応答伝文(データ部)ACK受信
                                        m_clMngMsgIn.m_clResDataAck.m_byAck = await ReadPipeWithCancel(m_clPipeIn);
                                    
                                        if (m_clMngMsgIn.m_clResDataAck.m_byAck.Length > 0)
                                        {
                                            // 応答伝文(データ部)ACK読み込み成功
                                            enStagePipe = (int)StagePipe.JudgeResDataAck;   // ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 応答伝文(データ部)ACK読み込み失敗
                                            enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ　
                                        }
                                    }
                                    catch (IOException ex)// 「応答伝文(データ部)ACK受信」で例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.RecvResDataAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 応答伝文(データ部)ACK判定
                                // ============================================================
                                case (int)StagePipe.JudgeResDataAck:
                                    try
                                    {
                                        // 応答伝文(データ部)ACKをデシリアライズする
                                        m_clMngMsgIn.m_clResDataAck.DeserializeAck();
                                        // 応答伝文(データ部)ACKの判定
                                        inMsgResult = m_clMngMsgIn.m_clResDataAck.JudgeAck();
                                        if (inMsgResult != 0)
                                        {
                                            // 応答伝文(データ部)ACK判定結果NG
                                            LOG(srLogPrefix + "Unmacth ResDataAck [" + inMsgResult.ToString() + "]");
                                        }
                                        inStageCnt = m_inStagePipeCntMax;           // ▼処理ステージ終了
                                    }
                                    catch (IOException ex)//「応答伝文(データ部)ACK受信」で例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.JudgeResDataAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    break;

                                // ============================================================
                                // 切断
                                // ============================================================
                                case (int)StagePipe.Disconnect:
                                    try
                                    {
                                        LOG(srLogPrefix + "Disconnect pipe ...");
                                        m_clPipeIn.Disconnect();
                                        ClosePipe(ref m_clPipeIn);
                                        inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                    }
                                    catch (IOException ex)                              // 例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.Disconnect クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                    }
                                    break;
                                    
                                // ============================================================
                                // 想定外の処理ステージ
                                // ============================================================
                                default:
                                    LOG(srLogPrefix + "Unknown stage[" + enStagePipe.ToString() + "]");
                                    inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                    break;
                            }
                        }

                        STATE.Error = (result != 0);    // エラーインジケータを更新

                    }
                    finally
                    {
                        //Releaseはトリガ処理で行うためここでは呼ばない
                    }
                }
            }
            catch (Exception ex)
            {
                LOG(srLogPrefix + " [FATAL] " + System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Edgeシステムからのデータ出力
        /// [TODO] : 必要に応じて値を読み込むタグを指定
        /// </summary>
        /// <remarks>
        /// このコンテナでは、
        /// クライアントから接続または送信されるまで待機するため、
        /// 入力/出力スレッドに対して、トリガ通信しない。
        /// 入力/出力スレッドは実行状態を判断して処理を行う
        /// </remarks>
        public async void ThreadOutService(CancellationToken cancelToken)
        {
            // ログ出力時に付加するサービス名(タスク名)
            string srLogPrefix = "[OutService]";
            bool ConnectSuccessFlag = true;
            try
            {
                int result = 0;
                while (true)
                {
                    // キャンセルトークンが来たら終了
                    if (cancelToken.IsCancellationRequested)
                        break;

                    // 実行状態が停止の場合は、ループの先頭へ
                    if (false == STATE.Run)
                    {
                        Thread.Sleep((int)STATE.Cycle);
                        continue;
                    }

                    //タスク処理を開始する
                    try
                    {
                        // 処理ステージの初期化
                        int enStagePipe = (int)StagePipe.Create;
                        // 戻り値
                        UInt32 dwRet = 0;
                        // 要求伝文判定結果
                        int inMsgResult = 0;
                        // 読み込みサイズ
                        int inReadSize = 0;

                        int edgeResult = 0;

                        // ============================================================
                        // 処理ステージを移動しながら、処理を行う
                        // ============================================================
                        for (int inStageCnt = 0; inStageCnt < m_inStagePipeCntMax; inStageCnt++)
                        {
                            // ============================================================
                            // 該当処理の実行
                            // ============================================================
                            switch (enStagePipe)
                            {
                                // ============================================================
                                // 名前付きパイプクラスの生成
                                // ============================================================
                                case (int)StagePipe.Create:
                                    try
                                    {
                                        // 名前付きパイプクラスの生成判定
                                        if (null == m_clPipeOut)
                                        {
                                            LOG(srLogPrefix + "Create pipe ...");
                                            // 名前付きパイプクラスの生成
                                            dwRet = CreatePipe(HMI_Static.HMI_PIPE_OUT_NAME, ref m_clPipeOut);
                                        }
                                        else
                                        {
                                            // 生成済みは正常終了と判断
                                            dwRet = 0;
                                        }
                                        // 生成結果判定
                                        if (0 == dwRet)
                                        {
                                            // 生成成功
                                            enStagePipe = (int)StagePipe.Connect;          // ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 生成失敗
                                            LOG(srLogPrefix + "Failed: create pipe[" + dwRet.ToString("X8") + "]");
                                            inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                        }
                                    }
                                    catch (IOException ex)// 例外が発生した際の処理
                                    {
                                        LOG(srLogPrefix + "Failed: StagePipe.Create Fatal. ex = " + ex.Message);
                                        inStageCnt = m_inStagePipeCntMax;
                                    }
                                    break;

                                // ============================================================
                                // クライアントからの接続
                                // ============================================================
                                case (int)StagePipe.Connect:
                                    // クライアント接続状態判定
                                    ConnectSuccessFlag = true;
                                    try
                                    {
                                        if (!m_clPipeOut.IsConnected)
                                        {
                                            LOG(srLogPrefix + "Wait for client connection ...");
                                            // クライアント接続待ち
                                            try
                                            {
                                                await m_clPipeOut.WaitForConnectionAsync(cancelToken);
                                                LOG(srLogPrefix + "Client connected.");
                                            }
                                            catch (IOException ex)
                                            {
                                            	Console.WriteLine("クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                                ConnectSuccessFlag = false;
                                            }

                                        }
                                    }
                                    catch (IOException ex)// 例外が発生した際の処理
                                    {
                                        ConnectSuccessFlag = false;
                                        LOG(srLogPrefix + "Failed: StagePipe.Connect Fatal. ex = " + ex.Message);
                                    }
                                    if (ConnectSuccessFlag == true)
                                        enStagePipe = (int)StagePipe.RecvReqHead;           // ▼次のステージへ
                                    else
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    
                                    break;

                                // ============================================================
                                // クライアントからの要求伝文(ヘッダ部)受信
                                // ============================================================
                                case (int)StagePipe.RecvReqHead:
                                    try
                                    {
                                        //要求伝文(ヘッダ部)をシリアライズする
                                        m_clMngMsgOut.m_clReqHead.SerializeHead();
                                        // 要求伝文(ヘッダ部)の読み込み
                                        //inReadSize = await m_clPipeIn.ReadAsync(m_clMngMsgIn.m_clReqHead.m_byHead, 0, m_clMngMsgIn.m_clReqHead.m_byHead.Length, cancelToken);
                                        m_clMngMsgOut.m_clReqHead.m_byHead = await ReadPipeWithCancel(m_clPipeOut);
                                        if (m_clMngMsgOut.m_clReqHead.m_byHead.Length > 0)
                                        {
                                            // 読み込み成功
                                            enStagePipe = (int)StagePipe.JudgeReqHead;        // ▼次のステージへ
                                        }
                                        else
                                        {
                                            // 読み込み失敗
                                            LOG(srLogPrefix + "Unmatch req head size [" + m_clMngMsgOut.m_clReqHead.m_byHead.Length.ToString() + "][" + inReadSize.ToString() + "] _ RecvReqHead Out");
                                            enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                        }
                                    }
                                    catch (IOException ex)// 例外が発生した際の処理
                                    {
                                         Console.WriteLine("StagePipe.RecvReqHead クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    break;

                                // ============================================================
                                // クライアントからの要求伝文(ヘッダ部)判定
                                // ============================================================
                                case (int)StagePipe.JudgeReqHead:
                                    try
                                    {
                                        // 要求伝文(ヘッダ部)をデシリアライズする
                                        m_clMngMsgOut.m_clReqHead.DeserializeHead();
                                        // 要求伝文(ヘッダ部)を判定する
                                        inMsgResult = m_clMngMsgOut.m_clReqHead.JudgeHead();
                                        if (inMsgResult != 0)
                                        {
                                            // 要求伝文(ヘッダ部)判定結果NG
                                            LOG(srLogPrefix + "Unmatch ReqHead [" + inMsgResult.ToString() + "]");
                                        }
                                        enStagePipe = (int)StagePipe.SendReqHeadAck; // ▼次のステージへ
                                    }
                                    catch (IOException ex)// 例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.JudgeReqHead クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    break;

                                // ============================================================
                                // 要求伝文(ヘッダ部)ACK送信
                                // ============================================================
                                case (int)StagePipe.SendReqHeadAck:
                                    try
                                    {
                                        // 要求伝文(ヘッダ部)ACK作成
                                        m_clMngMsgOut.m_clReqHeadAck.UpdateAck(m_clMngMsgOut.m_clReqHead.m_clHead.m_woMsgNo, (UInt32)inMsgResult);
                                        // 要求伝文(ヘッダ部)ACKシリアライズ
                                        m_clMngMsgOut.m_clReqHeadAck.SerializeAck();
                                        // 要求伝文(ヘッダ部)ACK送信
                                        await m_clPipeOut.WriteAsync(m_clMngMsgOut.m_clReqHeadAck.m_byAck, 0, m_clMngMsgOut.m_clReqHeadAck.m_byAck.Length, cancelToken);
                                        // 要求伝文(ヘッダ部)判定結果によって次のステージを決定
                                        if (inMsgResult == 0)
                                        {
                                            // 要求伝文(ヘッダ部)ACK判定結果OK
                                            // メッセージNoによる分岐
                                            switch (m_clMngMsgOut.m_clReqHead.m_clHead.m_woMsgNo)
                                            {
                                                case (UInt16)HMI_Msg.MsgNo.ReqWriteTag:
                                                    //タグデータ読み込み要求
                                                    enStagePipe = (int)StagePipe.RecvReqDataWriteTag;  // ▼次のステージへ
                                                    break;
                                                default:
                                                    // 上記以外はエラー
                                                    LOG(srLogPrefix + "Unmknown MsgNo [" + m_clMngMsgOut.m_clReqHead.m_clHead.m_woMsgNo.ToString() + "]");
                                                    inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                                    break;
                                            }
                                        }
                                        else
                                        {
                                            // 要求伝文(ヘッダ部)ACK判定結果NG
                                            inStageCnt = m_inStagePipeCntMax;           // ▼処理ステージ終了
                                        }
                                    }
                                    catch (IOException ex)// 例外が発生した際の処理
                                    {
                                        // 要求伝文(ヘッダ部)判定結果NG
                                          Console.WriteLine("StagePipe.SendReqHeadAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    break;

                                // ============================================================
                                // クライアントからの要求伝文(データ部)受信：タグ書き込み
                                // ============================================================
                                case (int)StagePipe.RecvReqDataWriteTag:
                                    try
                                    {
                                        // 要求伝文(データ部)の確保
                                        m_clMngMsgWriteTag.AllocReqData(m_clMngMsgOut.m_clReqHead.m_clHead.m_dwDataSize);
                                        // 要求伝文(データ部)の読み込み
                                        m_clMngMsgWriteTag.m_byReqData = await ReadPipeWithCancel(m_clPipeOut);
                                        // 要求伝文(データ部)デシリアライズ
                                        m_clMngMsgWriteTag.DeserializeReqData();
                                        // RT-edgeの書き込み
                                        object objType = new object();
                                        foreach (MsgDataTagValue tagvalue in m_clMngMsgWriteTag.m_clReqData.m_lsTags)
                                        {
                                            edgeResult = EGAPI.EgTagGetProperty(tagvalue.m_srName, "Type", ref objType);   //タグのTypeを取得しMsgDataTagValueに格納
                                            if(edgeResult == 0)
                                            {
                                                tagvalue.m_woType = (UInt16)objType;
                                                edgeResult = EGAPI.EgTagWrite(tagvalue.m_srName, tagvalue.GetConvertObjValue());
                                            }
                                            if (edgeResult != 0)
                                                result |= edgeResult;

                                        }

                                        if (m_clMngMsgWriteTag.m_byReqData.Length > 0)
                                        {
                                            // 書き込み成功
                                            inMsgResult = 0;
                                            m_clMngMsgWriteTag.SerializeReqData();
                                            enStagePipe = (int)StagePipe.SendReqDataAck; // ▼次のステージへ
                                        }
                                        else 
                                        {
                                            // 書き込み失敗
                                            inMsgResult = (int)HMI_Msg.ErrCode.UnmatchDataSize;
                                            LOG(srLogPrefix + "Unmatch req data size [" + m_clMngMsgReadTag.m_byReqData.Length.ToString() + "][" + inReadSize.ToString() + "]");
                                            enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                        }
                                    }
                                    catch (IOException ex)// 例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.RecvReqDataWriteTag クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }

                                break;

                                // ============================================================
                                // 要求伝文(データ部)ACK送信
                                // ============================================================
                                case (int)StagePipe.SendReqDataAck:
                                    try
                                    {
                                        // 要求伝文(データ情報)ACK作成
                                        m_clMngMsgOut.m_clReqDataAck.UpdateAck(m_clMngMsgOut.m_clReqHead.m_clHead.m_woMsgNo, (UInt32)inMsgResult);
                                        // 要求伝文(データ情報)ACKシリアライズ
                                        m_clMngMsgOut.m_clReqDataAck.SerializeAck();
                                        // 要求伝文(データ情報)ACK送信
                                        await m_clPipeOut.WriteAsync(m_clMngMsgOut.m_clReqDataAck.m_byAck, 0, m_clMngMsgOut.m_clReqDataAck.m_byAck.Length, cancelToken);

                                        inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了

                                    }
                                    catch (IOException ex)// 例外が発生した際の処理
                                    {
                                        Console.WriteLine("StagePipe.SendReqDataAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                        enStagePipe = (int)StagePipe.Disconnect;        // ▼次のステージへ
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    break;

                                

                                // ============================================================
                                // クライアントとの切断
                                // ============================================================
                                case (int)StagePipe.Disconnect:
                                    try
                                    {
                                        LOG(srLogPrefix + "Disconnect pipe ...");
                                        m_clPipeOut.Disconnect();
                                        ClosePipe(ref m_clPipeOut);
                                    }
                                    catch (IOException ex) //例外処理
                                    {
                                        Console.WriteLine("StagePipe.SendReqDataAck クライアントが切断しました。再接続を待機します。: " + ex.Message);
                                    }
                                    catch (Exception ex)
                                    {
                                        Console.WriteLine("エラーが発生しました: " + ex.Message); // その他のエラー処理
                                    }
                                    inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                    break;

                                // ============================================================
                                // 想定外の処理ステージ
                                // ============================================================
                                default:
                                    LOG(srLogPrefix + "Unknown stage[" + enStagePipe.ToString() + "]");
                                    inStageCnt = m_inStagePipeCntMax;               // ▼処理ステージ終了
                                    break;
                            }
                        }

                        STATE.Error = (result != 0);	// エラーインジケータを更新
                    }
                    finally
                    {
                        //Releaseはトリガ処理で行うためここでは呼ばない
                    }
                }
            }
            catch (Exception ex)
            {
                LOG(srLogPrefix + System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// 入力サービスを1回キックする
        /// 必要が無ければ変更不要
        /// </summary>
        public void KickInput()
        {
            try
            {
                STATE.hInTrigSem.Release();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// 出力サービスを1回キックする
        /// 必要が無ければ変更不要
        /// </summary>
        public void KickOutput()
        {
            try
            {
                STATE.hOutTrigSem.Release();
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// サービスタグ生成 ※すでに生成されている場合は無視して継続します
        /// [TODO] : 必要に応じて作成するタグを指定(タグの生成はXMLに定義し自動生成することを推奨)
        /// </summary>
        /// <returns></returns>
        public int CreateServiceTags()
        {
            try
            {
                int result = 0;
                // サービスタグ類の生成
                // [TODO] : 必要に応じて以下コードを修正
                //result |= EGAPI.EgTagCreate(EGCONST.TAG_MYSERV_XXX, (ushort)EGDEFINE.egTagDataType.UInt64, "", "sample tag");
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスインジケータの更新 ※更新に失敗しても無視して継続します
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        public int UpdateIndicator()
        {
            try
            {
                // インジケータタグはフレームワークによって自動生成済
                int result = 0;
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_STATUS, STATE.State);   // サービス起動状態
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_RUN, STATE.Run);        // サービス実行状態
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_LIVE, STATE.Live);      // サービス実行カウンタ
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_ERROR, STATE.Error);    // サービス異常状態
                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスプロパティの取得 ※取得できない時はデフォルト設定値を採用します
        /// [TODO] : 必要に応じて設定項目の増減指定
        /// </summary>
        /// <returns></returns>
        public int GetProperty()
        {
            try
            {
                // プロパティタグはXML定義可能　無いときはデフォルト設定値を採用します
                int result = 0;
                object value = new object();
                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_MODE, ref value);    // サービス動作モードの取得
                if (result == 0)
                    STATE.Mode = (byte)value;
                else
                    STATE.Mode = EDGECONST.DEFAULT_MODE;				            // デフォルト

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_CYCLE, ref value);   // サービス処理サイクルの取得
                if (result == 0)
                    STATE.Cycle = (uint)value;
                else
                    STATE.Cycle = EDGECONST.DEFAULT_CYCLE;				            // デフォルト

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_INPRIO, ref value);  // 入力サービススレッドのプライオリティ指定
                if (result == 0)
                    STATE.InPriority = (byte)value;
                else
                    STATE.InPriority = EDGECONST.DEFAULT_INPRIO;                    // デフォルト

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_INPRIO, ref value);  // 出力サービススレッドのプライオリティ指定
                if (result == 0)
                    STATE.OutPriority = (byte)value;
                else
                    STATE.OutPriority = EDGECONST.DEFAULT_OUTPRIO;                  // デフォルト

                //
                // [TODO] : 任意のサービスプロパティを指定
                //
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス開始 ※入出力可能になります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        public int StartService()
        {
            try
            {
                STATE.Run = true;       // サービス実行
                UpdateIndicator();      // サービスインジケータ更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス一時停止 ※入出力されなくなります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns></returns>
        public int PauseService()
        {
            try
            {
                STATE.Run = false;      // サービス一時停止
                UpdateIndicator();      // サービスインジケータ更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービスの初期化
        /// </summary>
        /// <returns>正常時０</returns>
        public Int32 InitService()
        {
            try
            {
                Int32 result = 0;
                LOG("Start HMI_Serv Service");
                edge_API.EDGECONFIG config = new edge_API.EDGECONFIG(); //設定情報の初期化
                config = EGAPI.EDGE_CONFIG_DEFAULT;                     //初期値設定

                config.mInstMessFunc = MsgHandler;                      //Edgeメッセージユーザハンドラの指定

                result = EGAPI.EgInit(config);                          // Edgeフレームワークの初期化
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                EGCONST.SET_EDGECONST(EGAPI.EGFW.ServiceRealName);      // 各タグ名生成

                result = GetProperty();                                 // サービスプロパティのロード処理
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                result = CreateServiceTags();                           // サービスインジケータの生成
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                STATE.hInTrigSem = new SemaphoreSlim(1, 1);             // 入力サービススレッド用トリガセマフォ
                STATE.hOutTrigSem = new SemaphoreSlim(1, 1);            // 出力サービススレッド用トリガセマフォ

                mThreadInServiceTS = new CancellationTokenSource();     //スレッド終了用token指定
                CancellationToken cInToken = mThreadInServiceTS.Token;    
                ThreadInService(cInToken);                              //入力スレッド処理作成始

                mThreadOutServiceTS = new CancellationTokenSource();    //スレッド終了用token指定
                CancellationToken cOutToken = mThreadOutServiceTS.Token;
                ThreadOutService(cOutToken);                            //出力スレッド処理作成始

                mThreadAutoTS = new CancellationTokenSource();          //スレッド終了用token指定
                CancellationToken cAutoToken = mThreadAutoTS.Token;
                ThreadAutoCycle(cAutoToken);                            //自動更新スレッド処理作成始

                STATE.State = EDGECONST.SERVICE_UP;                     // サービスアップ

                return result;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// サービス削除（プロセス停止）
        /// </summary>
        /// <returns>ん</returns>
        public int KillService()
        {
            try
            {
                PauseService();                         // サービス一時停止

                STATE.State = EDGECONST.SERVICE_DOWN;   //
                UpdateIndicator();				        // サービスインジケータ更新
                LOG("End " + EGCONST.SERVICENAME + " Service");
                return 0;
            }
            catch (Exception ex)
            {
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        /// <param name="filePath"></param>
        /// <param name="lineNumber"></param>
        public void WARN(int reason,
                         [CallerFilePath] string filePath = "",
                         [CallerLineNumber] int lineNumber = 0)
        {
            try
            {
                Console.WriteLine(EGCONST.SERVICENAME + " service warning at file(" + filePath + ") ,line(" + lineNumber.ToString() +
                                   ")  ,reason 0x" + reason.ToString("x8"));
            }
            catch
            { }
        }
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        public void LOG(string message)
        {
            try
            {
                // サービス名の設定
                string srName = "HMI_Serv";                     // デフォルト値のセット
                if (null != EGCONST.SERVICENAME)
                {
                    if ("" !=  EGCONST.SERVICENAME)
                    {
                        srName = EGCONST.SERVICENAME;
                    }
                }

                Console.WriteLine("[LOG]" + srName + " : " + message);

                // EgLogへ出力
                PrintLog(srName, edge_FW.EG_LOG_DBG, message);
            }
            catch
            { }
        }

        /// <summary>
        /// EgLogへ出力
        /// </summary>
        /// <param name="srName">サービス名</param>
        /// <param name="srMode">モード</param>
        /// <param name="srMsg">メッセージ</param>
        public void PrintLog(string srName, string srMode, string srMsg)
        {
            // 各変数が初期化されいない場合は終了
            if (null == EGAPI) { return; }
            if (null == EGAPI.EGFW) { return; }

            edge_FW._C_EgFW_PrintLog(srName, srMode, srMsg);
        }


        /// <summary>
        /// 名前付きパイプの生成
        /// </summary>
        /// <param name="srName">パイプ名</param>
        /// <param name="clPipe">名前付きパイプ</param>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        public UInt32 CreatePipe(string srName, ref NamedPipeServerStream clPipe)
        {
            UInt32 dwRet = 0;

            try
            {
                var pipeSecurity = new PipeSecurity();
                pipeSecurity.AddAccessRule(new PipeAccessRule("Everyone", PipeAccessRights.FullControl, AccessControlType.Allow));

                clPipe = new NamedPipeServerStream( srName,
                                                    PipeDirection.InOut,               //双方向
                                                    1,                                 // インスタンスは1
                                                    PipeTransmissionMode.Byte,         // (DEFAULT) バイトストリームモード
                                                    PipeOptions.Asynchronous,          // (DEFAULT) 非同期モード
                                                    0,                                 // (DEFAULT) inBufferSize (デフォルト値 4096)
                                                    0,                                 // (DEFAULT) outBufferSize (デフォルト値 4096)
                                                    pipeSecurity);                     // パイプセキュリティ

            }
            catch (Exception ex)
            {
                dwRet = (UInt32)ex.HResult;
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }

            return dwRet;
        }

        /// <summary>
        /// 名前付きパイプのクローズ
        /// </summary>
        /// <returns>0=正常終了/それ以外=エラー</returns>
        private UInt32 ClosePipe(ref NamedPipeServerStream clPipe)
        {
            UInt32 dwRet = 0;

            try
            {
                if (null != clPipe)
                {
                    clPipe.Close();
                }
                clPipe = null;
            }
            catch (Exception ex)
            {
                dwRet = (UInt32)ex.HResult;
                LOG(System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }

            return dwRet;
        }

    }
}
